//= require <dSS/SaveSceneDialog>

/* globals dSS, dSA, Ext */

// Activities tab content

Ext.namespace('dSS', 'dSA');

dSS.AV_MAGIC_ZONE = NaN;

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function () {
        // this controls the single click expand option
        this.annoyingMode = true;
        this.activeNode = null;

        this.dirtyCount = 0;
        this.dirtyDevices = {};

        this.treePanel = new Ext.tree.TreePanel({
            cls: 'dsClsScenesPanel_TreePanel',

            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            ctCls: 'x-panel-no-top-border',
            root: new Ext.tree.AsyncTreeNode({
                cls: 'dsClsScenesPanel_TreePanel_Root'
            }),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        this.apartmentTreePanel = new Ext.tree.TreePanel({
            cls: 'dsClsScenesPanel_ApartmentTreePanel',

            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            root: new Ext.tree.AsyncTreeNode({
                cls: 'dsClsScenesPanel_ApartmentTreePanel_Root',

                children: this.generateApartmentNodes(),
                id: 'root',
                singleClickExpand: this.annoyingMode,
                leaf: false
            }),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.menuPanel = new Ext.Panel({
            cls: 'dsClsScenesPanel_MenuPanel',

            title: dSS._('Scene', 'Scenes', 2),
            region: 'west',
            layout: 'border',
            split: true,
            width: 225,
            minSize: 175,
            tbar: this.buildTopToolbar(),
            bodyBorder: false,
            items: [
                {
                    cls: 'dsClsScenesPanel_MenuPanel_ZonePanel',

                    xtype: 'panel',
                    ref: 'zoneMenuArea',
                    layout: 'fit',
                    region: 'center',
                    split: true,
                    items: [
                        this.treePanel
                    ]
                }, {
                    cls: 'dsClsScenesPanel_MenuPanel_ApartmentPanel',

                    xtype: 'panel',
                    ref: 'apartmentMenuArea',
                    layout: 'fit',
                    region: 'south',
                    // height: 80,
                    split: true,
                    items: [
                        this.apartmentTreePanel
                    ]
                }
            ]
        });

        this.saveText = new Ext.form.Label({
            cls: 'dsClsScenesPanel_SaveLabel',

            text: dSS._('The current settings can be saved for the selected scene.'),
            colspan: 3
        });

        this.discardButton = new Ext.Button({
            cls: 'dsClsScenesPanel_DiscardButton',

            text: dSS._('Discard'),
            disabled: true,
            minWidth: 100,
            handler: this.processSelectedScene.createDelegate(this, [ false ])
        });

        this.saveButton = new Ext.Button({
            cls: 'dsClsScenesPanel_SaveButton',

            text: dSS._('Save'),
            minWidth: 100,
            handler: this.saveSelectedScene.createDelegate(this)
        });

        this.saveToAnotherSceneButton = new Ext.Button({
            cls: 'dsClsScenesPanel_SaveToAnotherSceneButton',

            text: dSS._('Save to another scene'),
            handler: this.saveToAnotherScene.createDelegate(this),
            scope: this,
            colspan: 3,
            width: '100%'
        });

        var savePanel = new Ext.Panel({
            cls: 'dsClsScenesPanel_SavePanel',

            layout: 'table',
            border: false,
            scope: this,
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 58,
            layoutConfig: {
                columns: 3,
                tableAttrs: {
                    width: '100%'
                }
            },
            ref: 'saveScene',
            items: [
                this.saveText,
                {
                    xtype: 'box',
                    height: 10,
                    colspan: 3
                }, {
                    xtype: 'panel',
                    layout: 'hbox',
                    border: false,
                    height: 30,
                    items: [
                        this.saveButton,
                        {
                            xtype: 'box',
                            flex: 3
                        },
                        this.discardButton
                    ]
                }, {
                    xtype: 'box',
                    height: 10,
                    colspan: 3
                },
                this.saveToAnotherSceneButton
            ]
        });

        this.controlPanel = new Ext.Panel({
            cls: 'dsClsScenesPanel_ControlPanel',

            region: 'east',
            ref: 'controlPanel',
            disabled: true,
            containerScroll: true,
            bodyCssClass: 'x-panel-no-bottom-border',
            autoScroll: true,
            tbar: {
                height: 26,
                items: [
                    {
                        xtype: 'tbtext',
                        text: '&nbsp;',
                        width: 17,
                        ref: 'icon'
                    }, {
                        xtype: 'tbtext',
                        cls: 'x-black-text x-text-align-left x-text-cut',
                        ref: 'message'
                    }
                ],
                cls: 'x-panel-no-top-border'
            },
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 20,
            items: [
                {
                    cls: 'dsClsScenesPanel_ControlPanel_DeviceSettingsPanel',

                    xtype: 'dssdevicesettingsrootpanel',
                    border: false,
                    ref: 'settings',
                    embedded: true,
                    devices: [],
                    index: -1
                }, {
                    cls: 'dsClsScenesPanel_ControlPanel_SaveScenePanel',

                    xtype: 'panel',
                    ref: 'saveScenePanel',
                    padding: 10,
                    // border: false,
                    bodyCssClass: 'x-panel-no-right-border ' +
                                  'x-panel-no-left-border ' +
                                  'x-panel-no-bottom-border',
                    items: [ savePanel ]
                }
            ]
        });

        var contentPanel = {
            cls: 'dsClsScenesPanel_ContentPanel',

            ref: 'contentPanel',
            region: 'center',
            title: dSS._('Device', 'Devices', 2),
            bodyCssClass: 'x-panel-no-right-border',
            layout: 'border',
            items: [
                {
                    xtype: 'dssdevicepanel',
                    ref: 'devicePanel',
                    border: false,
                    frame: false,
                    region: 'center',
                    featureConfig: dSS.config.FeatureConfig.SCENE
                },
                this.controlPanel
            ]
        };

        Ext.apply(this, {
            itemId: 'activities',
            cls: 'dsClsScenesPanel',

            layout: 'border',
            border: false,
            items: [ this.menuPanel, contentPanel ]
        });

        this.addEvents('treeredrawcomplte');
        this.dirtyTree = false;
        this.updateTree();

        dSS.data.Loader.getZoneStore().addListener('datachanged',
            function () {       // store
                this.dirtyTree = true;
                if (this.isVisible()) {
                    this.fireEvent('redrawtree');
                }
            }, this);

        dSS.data.Loader.getZoneStore().addListener('update',
            function () {       // store
                this.dirtyTree = true;
                if (this.isVisible()) {
                    this.fireEvent('redrawtree');
                }
            }, this);

        dSS.data.Loader.getSceneStore().addListener('datachanged',
            function () {       // store
                this.dirtyTree = true;
                if (this.isVisible()) {
                    this.fireEvent('redrawtree');
                }
            }, this);

        dSS.data.Loader.getSceneStore().addListener('update',
            function () {       // store
                this.dirtyTree = true;
                if (this.isVisible()) {
                    this.fireEvent('redrawtree');
                }
            }, this);

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function () {
            if (dSS.config.firstLoad === true) {
                var fn = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
                fn();
                dSS.data.Loader.load();
                dSS.config.firstLoad = false;
            }

            this.controlPanel.settings.setRowSelectionModel(
                this.contentPanel.devicePanel.getSelectionModel()
            );

            this.contentPanel.devicePanel.on('rowclick', function (grid) {
                var sel = grid.getSelectionModel();
                var rec = sel.getSelected();
                this.controlSaveButton(rec);
            }, this);

            this.contentPanel.devicePanel.on('sortchange', function (grid) {
                var devices = grid.getStore().getRange();
                if (devices.length) {
                    this.controlPanel.settings.loadDevices(devices);
                }
            }, this);

            this.contentPanel.devicePanel.getSelectionModel().on(
                'selectionchange', function (sel) {
                    var rec = sel.getSelected();
                    this.controlSaveButton(rec);
                }, this
            );
        }, this);

        this.on('activate', function () {
            if (this.activeNode) {
                this.activeNode.setCls();
                this.activeNode = null;
            }
            this.handleTreeView();

            if (this.menuSizeSet !== true) {
                var total = this.menuPanel.ownerCt.getHeight();
                this.menuPanel.apartmentMenuArea.setHeight(total / 3);
                this.menuPanel.zoneMenuArea.setHeight(total / 3 * 2);
                this.menuPanel.syncSize();
                this.menuSizeSet = true;
            }
        }, this);

        this.on('deactivate', function () {
            this.dirtyCount = 0;
            this.dirtyDevices = {};
        }, this);

        this.controlPanel.settings.on('valuechanged', function (dsuid) {
            if (!this.dirtyDevices[dsuid]) {
                this.dirtyCount++;
                this.dirtyDevices[dsuid] = true;
            }
            if (this.dirtyCount > 0) {
                this.discardButton.enable();
            }
            var node = this.getSelectedNode();
            if (!node) {
                return;
            }
            var ids = this.getZoneSceneGroup(node.id);
            var data = this.getNodeDisplayData(ids.zoneId, ids.groupId,
                ids.sceneNumber);
            this.setSaveText(data.name, this.dirtyCount);
            if (node.isLeaf()) {
                node.setCls();
            }
            if (this.activeNode) {
                this.activeNode.setCls();
                this.activeNode = null;
            }
        }, this);

        this.controlPanel.settings.on('updatetitleicon', function (icon) {
            var tbar = this.controlPanel.getTopToolbar();
            if (icon === 'loading-indicator-small') {
                tbar.message.setText(dSS._('Loading settings') + '...');
                tbar.icon.addClass('loading-indicator loading-indicator-small');
            } else {
                tbar.message.setText('&nbsp;');
                tbar.icon.removeClass('loading-indicator');
                tbar.icon.removeClass('loading-indicator-small');
            }
        }, this);

        this.on('redrawtree', this.handleTreeView.createDelegate(this), this, { buffer: 1000 });

        dSS.MessageBus.on('updatesavesceneandsettingspanel',
            this.updateSaveSceneAndSettingsPanel.createDelegate(this), this);

        dSS.MessageBus.on('scenesmarkdirty',
            this.markDirty.createDelegate(this), this);

        dSS.MessageBus.on('reloadapartment',
            dSS.util.maskViewport.createDelegate(this, [ 'mask' ]), this);

        dSS.MessageBus.on('structureloaded',
            dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]), this);

        dSS.MessageBus.on('reloaddevicescenevalue', function () {
            var sm = this.contentPanel.devicePanel.getSelectionModel();
            if (sm.hasSelection()) {
                this.controlPanel.settings.updateView(false);
            } else {
                this.controlPanbel.settings.clearDialog();
            }
        }, this);

        this.treePanel.getSelectionModel().on('beforeselect', function (sel, node) {        // , oldnode
            if (node.isLeaf() && node.disabled) {
                return false;
            }
            this.apartmentTreePanel.getSelectionModel().clearSelections();
        }, this);

        this.treePanel.on('contextmenu', this.handleContextMenu, this);

        this.apartmentTreePanel.on('contextmenu', this.handleContextMenu, this);

        this.apartmentTreePanel.getSelectionModel().on('beforeselect', function () {        // tree, node
            this.treePanel.getSelectionModel().clearSelections();
        }, this);

    },

    controlSaveButton: function (rec) {
        this.saveButton.enable();
        this.saveToAnotherSceneButton.enable();
        if (rec) {
            var arr = rec.get('groups');
            if (arr.length) {
                if (arr[0] === dSS.config.groups.TEMPCONTROL) {
                    this.saveButton.disable();
                    this.saveToAnotherSceneButton.disable();
                }
            }
        }
    },

    setSaveText: function (sceneName, dirtyDevicesCount) {
        if (dirtyDevicesCount > 0) {
            this.saveText.update(dSS.gt.strargs(
                dSS._('The values for scene %1 were modified on %2 ' +
                      'device and have not yet been saved. What would ' +
                      'you like to do with current settings?',
                'The values for scene %1 were modified on %2 ' +
                      'devices and have not yet been saved. What would ' +
                      'you like to do with current settings?', dirtyDevicesCount),
                [ '&quot;' + sceneName + '&quot;', dirtyDevicesCount ]
            ));
        } else {
            this.saveText.update(dSS.gt.strargs(
                dSS._('The current settings can be saved for the selected scene (%1).'),
                [ sceneName ]
            ));
        }
    },

    handleContextMenu: function (node, event) {
        event.preventDefault();
        event.stopEvent();

        if (!node.isSelected()) {
            node.select();
            this.handleNodeClick(node);
        }

        if (!node.isLeaf() || node.disabled || node.attributes.nocontextmenu) {
            return;
        }

        if (!this.contextMenu) {
            this.contextMenu = new Ext.menu.Menu({
                cls: 'dsClsScenesPanel_ContextMenu'
            });
        } else {
            this.contextMenu.removeAll();
        }

        var menuItem = new Ext.menu.Item({
            cls: 'dsClsScenesPanel_ContextMenu_CallSceneItem',

            text: dSS._('Call scene'),
            iconCls: 'callSceneIcon',
            handler: this.processScene.createDelegate(this, [ node, false ])
        });
        this.contextMenu.add(menuItem);

        if (node.attributes.undo) {
            menuItem = new Ext.menu.Item({
                cls: 'dsClsScenesPanel_ContextMenu_UndoSceneItem',

                text: dSS._('Undo scene'),
                iconCls: 'undoSceneIcon',
                handler: this.processScene.createDelegate(this, [ node, true ])
            });
            this.contextMenu.add(menuItem);
        }

        var path = node.getPath().split('/').slice(1);
        if (node.attributes.rename || path.length === 4) {
            var ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId != null) {
                this.contextMenu.add('-');
                menuItem = new Ext.menu.Item({
                    cls: 'dsClsScenesPanel_ContextMenu_RenameSceneItem',

                    text: dSS._('Rename scene'),
                    iconCls: 'editIcon',
                    handler: this.renameSelectedScene.createDelegate(this, [node])
                });
                this.contextMenu.add(menuItem);
            }
        }

        var zoneId = parseInt(path[1], 10);
        if (zoneId === dSS.config.zones.BROADCAST) {
            this.contextMenu.add('-');
            menuItem = new Ext.menu.Item({
                cls: 'dsClsScenesPanel_ContextMenu_SaveSceneItem',

                text: dSS._('Save scene'),
                iconCls: 'saveScene',
                handler: this.saveSelectedScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);
        }

        var xy = event.getXY();
        this.contextMenu.showAt(xy);
    },

    markDirty: function () {
        this.dirtyTree = true;
        if (this.isVisible()) {
            this.fireEvent('redrawtree');
        }
    },

    handleTreeView: function () {
        if (this.dirtyTree) {
            this.updateTree();
            return;
        }

        var selModel = this.treePanel.getSelectionModel();
        var node = selModel.getSelectedNode();
        var root = this.treePanel.getRootNode();

        if (node == null) {
            selModel = this.apartmentTreePanel.getSelectionModel();
            node = selModel.getSelectedNode();
        }
        if (node == null) {
            if (root.hasChildNodes()) {
                root.firstChild.select();
                this.handleNodeClick(root.firstChild);
            } else {
                this.updateTree();
            }
        } else {
            this.handleNodeClick(node);
        }
    },

    getSceneName: function (zone, group, scene) {
        var sceneStore = dSS.data.Loader.getSceneStore();
        var rec = sceneStore.getById(zone + '-' + group + '-' + scene);
        if (!rec) {
            return '';
        }

        var name = rec.get('name');
        if (name == null || !name.length) {
            return '';
        }

        return ' - ' + name;
    },

    getNodeDisplayData: function (zone, group, scene) {
        var name = dSS.config.sceneNames[scene];
        var tip = dSS.config.sceneTips[scene];
        if (group === dSS.config.groups.SHADOW) {
            if (dSS.config.sceneNamesShadow[scene] != null) {
                name = dSS.config.sceneNamesShadow[scene];
            }
            if (dSS.config.sceneTipsShadowOverrides[scene] != null) {
                tip = dSS.config.sceneTipsShadowOverrides[scene];
            }
        } else if (
            group === dSS.config.groups.RECIRCULATION ||
           group === dSS.config.groups.VENTILATION ||
           group === dSS.config.groups.APT_VENTILATION
       ) {
            if (dSS.config.sceneNamesAV[scene] != null) {
                name = dSS.config.sceneNamesAV[scene];
            }
            if (dSS.config.sceneTipsVentilationOverrides[scene] != null) {
                tip = dSS.config.sceneTipsVentilationOverrides[scene];
            }
        }

        var text = dSS._(name) + this.getSceneName(zone, group, scene);
        return { name: text, tip: dSS._(tip) };
    },

    generateSceneNode: function (zone, group, scene, icon, locks) {
        var node = {};
        var data = this.getNodeDisplayData(zone, group, scene);
        var name = data.name;
        var tip = data.tip;

        var locked = false;
        if (locks.indexOf(scene) !== -1) {
            locked = true;
        }

        node.text = name;
        node.id = zone + '-' + group + '-' + scene;
        node.leaf = true;
        node.iconCls = icon;
        node.qtip = tip;
        node.lockedScene = locked;
        return node;
    },

    // generates scenes 0-4 for a given zone/group
    generate0to4Scenes: function (zone, group, icon, locks) {
        var addScenes = [ dSS.config.scenes.T0_S0 ];

        if (group === dSS.config.groups.LIGHT) {
            addScenes.push(dSS.config.scenes.AUTO_OFF);
        }

        addScenes = addScenes.concat([
            dSS.config.scenes.T0_S1, dSS.config.scenes.T0_S2,
            dSS.config.scenes.T0_S3, dSS.config.scenes.T0_S4
        ]);

        if (group === dSS.config.groups.SHADOW) {
            addScenes.push(dSS.config.scenes.SUNPROTECTION);
        }

        if (group === dSS.config.groups.VENTILATION || group === dSS.config.groups.RECIRCULATION) {
            addScenes.push(dSS.config.scenes.BOOST);
        }

        var activeScenes = zone.get('activeBasicScenes');
        var realScenes = [];
        if (activeScenes && activeScenes[group]) {
            Ext.each(addScenes, function (scene) {
                var index = activeScenes[group].indexOf(scene);
                if (index !== -1) {
                    realScenes.push(scene);
                }
            });
        } else {
            realScenes = addScenes;
        }

        var scenes = [];

        for (var i = 0; i < realScenes.length; i++) {
            var nodeIcon = icon;
            // off scenes have special icons
            if (realScenes[i] === dSS.config.scenes.T0_S0) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            } else if (realScenes[i] === dSS.config.scenes.AUTO_OFF) {
                nodeIcon = 'autoStandby';
            } else if (realScenes[i] === dSS.config.scenes.SUNPROTECTION) {
                nodeIcon = 'sunprotection';
            }
            scenes.push(
                this.generateSceneNode(zone.get('zoneId'), group, realScenes[i], nodeIcon, locks)
            );
        }
        return scenes;
    },

    addAreaScenesToList: function (filter, group) {
        if (!filter) {
            return false;
        }

        switch (group) {
            case dSS.config.groups.LIGHT:
            case dSS.config.groups.SHADOW:
                return true;
            default:
                break;
        }

        return false;
    },

    generatePushbuttonScenes: function (zone, group, icon, cfg, locks) {
        var scenes = [];
        var sceneNumbers = [];

        if (this.disableSceneFilter) {
            cfg = [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS.config.deviceParams.PUSHBUTTON_GROUP8
            ];
        } else {
            cfg = dSS.util.UniqueArray(cfg, true);
            if (!cfg.length) {
                return scenes;
            }
        }

        var offScenes = [
            dSS.config.scenes.EX5, dSS.config.scenes.EX10,
            dSS.config.scenes.EX15, dSS.config.scenes.EX20
        ];

        var i;
        for (i = 0; i < cfg.length; i++) {
            switch (cfg[i]) {
                case dSS.config.deviceParams.PUSHBUTTON_DEVICE: // device pushbutton - no extra nodes
                case dSS.config.deviceParams.PUSHBUTTON_ZONE0: // zone pushbutton, defaults scenes 0-4, no extra nodes
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
                    // mood area 1
                    if (this.addAreaScenesToList(this.disableSceneFilter, group)) {
                        offScenes.push(dSS.config.scenes.T1_S0);
                        sceneNumbers.push(dSS.config.scenes.T1_S0);
                    }
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers.push(dSS.config.scenes.T1_S1);
                    }
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
                    // mood area 2
                    if (this.addAreaScenesToList(this.disableSceneFilter, group)) {
                        offScenes.push(dSS.config.scenes.T2_S0);
                        sceneNumbers.push(dSS.config.scenes.T2_S0);
                    }
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers.push(dSS.config.scenes.T2_S1);
                    }
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
                    // mood area 3
                    if (this.addAreaScenesToList(this.disableSceneFilter, group)) {
                        offScenes.push(dSS.config.scenes.T3_S0);
                        sceneNumbers.push(dSS.config.scenes.T3_S0);
                    }
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers.push(dSS.config.scenes.T3_S1);
                    }
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
                    // mood area 4
                    if (this.addAreaScenesToList(this.disableSceneFilter, group)) {
                        offScenes.push(dSS.config.scenes.T4_S0);
                        sceneNumbers.push(dSS.config.scenes.T4_S0);
                    }
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers.push(dSS.config.scenes.T4_S1);
                    }
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                    // moods 5-9
                    sceneNumbers.push(dSS.config.scenes.EX5);
                    sceneNumbers.push(dSS.config.scenes.EX6);
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                    // moods 7-9
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers = [ dSS.config.scenes.T1_S1 ].concat(sceneNumbers);
                    }
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                    // moods 10-14
                    sceneNumbers.push(dSS.config.scenes.EX10);
                    sceneNumbers.push(dSS.config.scenes.EX11);
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                    // moods 12-14
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers = [ dSS.config.scenes.T2_S1 ].concat(sceneNumbers);
                    }
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                    // moods 15-19
                    sceneNumbers.push(dSS.config.scenes.EX15);
                    sceneNumbers.push(dSS.config.scenes.EX16);
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                    // moods 17-19
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers = [ dSS.config.scenes.T3_S1 ].concat(sceneNumbers);
                    }
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                    // moods 20-24
                    sceneNumbers.push(dSS.config.scenes.EX20);
                    sceneNumbers.push(dSS.config.scenes.EX21);
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                    // moods 22-24
                    if (this.addAreaScenesToList(true, group)) {
                        sceneNumbers = [ dSS.config.scenes.T4_S1 ].concat(sceneNumbers);
                    }
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                default:
                    break;
            }
        }

        sceneNumbers = dSS.util.UniqueArray(sceneNumbers, true);
        sceneNumbers.sort(function (a, b) {
            return dSS.config.sceneSortOrder[a] - dSS.config.sceneSortOrder[b];
        });

        for (i = 0; i < sceneNumbers.length; i++) {
            var nodeIcon = icon;
            if (offScenes.indexOf(sceneNumbers[i]) !== -1) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            }

            var prependSeparator = false;

            switch (sceneNumbers[i]) {
                case dSS.config.scenes.T1_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX6) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T2_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX11) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T3_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX16) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T4_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX21) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.EX5:
                case dSS.config.scenes.EX10:
                case dSS.config.scenes.EX15:
                case dSS.config.scenes.EX20:
                    prependSeparator = true;
                    break;

                default:
                    break;
            }

            if (prependSeparator) {
                scenes.push(this.generateSeparatorNode());
            }
            scenes.push(
                this.generateSceneNode(zone, group, sceneNumbers[i], nodeIcon, locks)
            );
        }
        return scenes;
    },

    getLockedScenes: function (clusterLocks, group) {
        var lockedScenes = [];
        Ext.each(clusterLocks, function (lock) {
            if (lock.deviceClass == group) {
                lockedScenes = lock.lockedScenes;
                return false;
            }
            return true;
        });

        return lockedScenes;
    },

    // for leafs the id is encoded as: zone id - group - scene number
    // when no group is desired, 0 is used
    generateSubNodes: function (zone, groups, clusterLocks) {
        var zoneId = zone.get('zoneId');
        var retNodes = [];
        var childNodes = [];
        var locks;

        if (groups.LIGHT) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.GE);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.LIGHT,
                'lightbulbOn',
                locks);
            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.LIGHT,
                'lightbulbOn',
                groups.LIGHTpb,
                locks
            ));
            retNodes.push({
                text: dSS._('Light scenes'),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.GE],
                children: childNodes
            });
        }

        if (groups.SHADOW) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.GR);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.SHADOW,
                'shadesIcon',
                locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.SHADOW,
                'shadesIcon',
                groups.SHADOWpb, locks
            ));

            retNodes.push({
                text: dSS._('Shade scenes'),
                id: zoneId + '-' + dSS.config.groups.SHADOW + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.GR],
                children: childNodes
            });
        }

        if (groups.HEATING) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.BL);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.HEATING,
                'climateIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.HEATING,
                'climateIcon',
                groups.HEATINGpb, locks
            ));

            retNodes.push({
                text: dSS._('Heating scenes'),
                id: zoneId + '-' + dSS.config.groups.HEATING + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL],
                children: childNodes
            });
        }

        if (groups.COOLING) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.BL);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.COOLING,
                'climateIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.COOLING,
                'climateIcon',
                groups.COOLINGpb, locks
            ));

            retNodes.push({
                text: dSS._('Cooling scenes'),
                id: zoneId + '-' + dSS.config.groups.COOLING + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL],
                children: childNodes
            });
        }

        if (groups.VENTILATION) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.BL);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.VENTILATION,
                'ventilationSceneOn', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.VENTILATION,
                'ventilationSceneOn',
                groups.VENTILATIONpb, locks
            ));

            retNodes.push({
                text: dSS._('Ventilation scenes'),
                id: zoneId + '-' + dSS.config.groups.VENTILATION + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL],
                children: childNodes
            });
        }

        if (groups.RECIRCULATION) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.BL);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.RECIRCULATION,
                'ventilationSceneOn', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.RECIRCULATION,
                'ventilationSceneOn',
                groups.RECIRCULATIONpb, locks
            ));

            retNodes.push({
                text: dSS._('Recirculation scenes'),
                id: zoneId + '-' + dSS.config.groups.RECIRCULATION + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL],
                children: childNodes
            });
        }

        if (groups.AUDIO === true) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.TK);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.AUDIO,
                'audioIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.AUDIO,
                'audioIcon',
                groups.AUDIOpb, locks
            ));

            retNodes.push({
                text: dSS._('Audio scenes'),
                id: zoneId + '-' + dSS.config.groups.AUDIO + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.TK],
                children: childNodes
            });
        }

        if (groups.VIDEO) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.MG);
            childNodes = this.generate0to4Scenes(zone,
                dSS.config.groups.VIDEO,
                'videoIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.VIDEO,
                'videoIcon',
                groups.VIDEOpb, locks
            ));

            retNodes.push({
                text: dSS._('Video scenes'),
                id: zoneId + '-' + dSS.config.groups.VIDEO + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.MG],
                children: childNodes
            });
        }

        if (groups.WHITE) {
            locks = this.getLockedScenes(clusterLocks, dSS.config.deviceClass.WE);
            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                zoneId,
                dSS.config.groups.SINGLEDEVICES,
                'whiteIcon',
                groups.WHITEpb, locks
            ));

            retNodes.push({
                text: dSS._('White goods scenes'),
                id: zoneId + '-' + dSS.config.groups.SINGLEDEVICES + '-all',
                nocontextmenu: true,
                noreadareaconfig: true,
                lockedScene: true,
                leaf: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.WE],
                children: childNodes
            });
        }

        var ls = zone.get('lockedApartmentScenes');
        retNodes = retNodes.concat([ {
            text: dSS._('Standby'),
            id: zoneId + '-all-' + dSS.config.scenes.STANDBY,
            leaf: true,
            iconCls: 'standByIcon',
            lockedScene: (ls.indexOf(dSS.config.scenes.STANDBY) !== -1)
        }, {
            text: dSS._('Deep off'),
            id: zoneId + '-all-' + dSS.config.scenes.DEEP_OFF,
            leaf: true,
            iconCls: 'deepOffIcon',
            lockedScene: (ls.indexOf(dSS.config.scenes.DEEP_OFF) !== -1)
        }, {
            text: dSS._('Sleeping'),
            id: zoneId + '-all-' + dSS.config.scenes.SLEEPING,
            leaf: true,
            iconCls: 'sleepingIcon',
            lockedScene: (ls.indexOf(dSS.config.scenes.SLEEPING) !== -1)
        }, {
            text: dSS._('Wake up'),
            id: zoneId + '-all-' + dSS.config.scenes.WAKE_UP,
            leaf: true,
            iconCls: 'wakeupIcon',
            lockedScene: (ls.indexOf(dSS.config.scenes.WAKE_UP) !== -1)
        } ]);

        return retNodes;
    },

    generateApartmentVentilationNodes: function (scenes) {
        var retNodes = [];
        var iconcss = 'ventilationScene';

        for (var i = 0; i < scenes.length; i++) {
            var name = dSS.config.sceneNamesAV[scenes[i]];
            if (name == null) {
                name = dSS.config.sceneNames[scenes[i]];
            }

            retNodes.push({
                text: dSS._(name),
                id: dSS.AV_MAGIC_ZONE + '-' + dSS.config.groups.APT_VENTILATION + '-' + scenes[i],
                iconCls: scenes[i] === 0
                    ? iconcss + 'Off'
                    : iconcss + 'On',
                leaf: true
            });
        }
        return retNodes;
    },

    generateApartmentNodes: function () {
        return [ {
            text: dSS._('Leave home'),
            id: '0' + '-all-' + dSS.config.scenes.ABSENT,
            leaf: true,
            iconCls: 'leaveHomeIcon'
        },
        {
            text: dSS._('Coming home'),
            id: '0' + '-all-' + dSS.config.scenes.PRESENT,
            leaf: true,
            iconCls: 'comeHomeIcon'
        },
        {
            text: dSS._('Door bell'),
            id: '0' + '-all-' + dSS.config.scenes.SIG_BELL,
            leaf: true,
            iconCls: 'doorBellIcon'
        },
        this.generateSeparatorNode(),
        {
            text: dSS._('Rain'),
            id: '0' + '-all-' + dSS.config.scenes.RAIN,
            iconCls: 'rainIcon',
            undo: false,
            leaf: true
        },
        {
            text: dSS._('No rain'),
            id: '0' + '-all-' + dSS.config.scenes.NO_RAIN,
            iconCls: 'noRainIcon',
            undo: false,
            leaf: true
        },
        {
            text: dSS._('Wind'),
            id: '0' + '-all-' + dSS.config.scenes.WIND,
            iconCls: 'windIcon',
            undo: false,
            leaf: true
        },
        {
            text: dSS._('No wind'),
            id: '0' + '-all-' + dSS.config.scenes.NO_WIND,
            iconCls: 'noWindIcon',
            undo: false,
            leaf: true
        },
        {
            text: dSS._('Hail'),
            id: '0' + '-all-' + dSS.config.scenes.HAIL,
            iconCls: 'hailIcon',
            undo: false,
            leaf: true
        },
        {
            text: dSS._('No hail'),
            id: '0' + '-all-' + dSS.config.scenes.NO_HAIL,
            iconCls: 'noHailIcon',
            undo: false,
            leaf: true
        },
        this.generateSeparatorNode(),
        {
            text: dSS._('Panic'),
            id: '0' + '-all-' + dSS.config.scenes.PANIC,
            iconCls: 'panicIcon',
            undo: true,
            leaf: true
        },
        {
            text: dSS._('Fire'),
            id: '0' + '-all-' + dSS.config.scenes.FIRE,
            iconCls: 'fireIcon',
            undo: true,
            leaf: true
        },
        {
            text: dSS._('Alarm 1'),
            id: '0' + '-0-' + dSS.config.scenes.ALARM1,
            iconCls: 'alarm1Icon',
            undo: true,
            rename: true,
            leaf: true
        },
        {
            text: dSS._('Alarm 2'),
            id: '0' + '-0-' + dSS.config.scenes.ALARM2,
            iconCls: 'alarm2Icon',
            undo: true,
            rename: true,
            leaf: true
        },
        {
            text: dSS._('Alarm 3'),
            id: '0' + '-0-' + dSS.config.scenes.ALARM3,
            iconCls: 'alarm3Icon',
            undo: true,
            rename: true,
            leaf: true
        },
        {
            text: dSS._('Alarm 4'),
            id: '0' + '-0-' + dSS.config.scenes.ALARM4,
            iconCls: 'alarm4Icon',
            undo: true,
            rename: true,
            leaf: true
        } ];
    },

    generateSeparatorNode: function () {
        return {
            text: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;',
            disabled: true,
            cls: 'x-tree-noicon x-item-disabled mousedefault',
            isTarget: false,
            allowDrag: false,
            allowDrop: false,
            isSeparator: true,
            leaf: true,
            listeners: {
                contextmenu: function () {
                    return false;
                }
            }
        };
    },

    updateApartmentSceneName: function (scene) {
        var node = this.apartmentTreePanel.getNodeById('0-0-' + String(scene));
        if (node) {
            node.setText(dSS._(dSS.config.sceneNames[scene]) + this.getSceneName(0, 0, scene));
        }
    },

    updateTree: function () {
        var devStore = dSS.data.Loader.getDeviceStore();
        devStore.clearFilter();

        var panel = this.apartmentTreePanel;
        var selModel = panel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = '';

        if (selectedNode == null) {
            panel = this.treePanel;
            selModel = panel.getSelectionModel();
            selectedNode = selModel.getSelectedNode();
        }

        if (selectedNode) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var nodes = [];
        var addApartmentVentilation = false;
        zStore.each(
            function (record) {
                if (record.get('zoneId') !== dSS.config.zones.BROADCAST && devStore.find('zoneId', record.get('zoneId')) >= 0) {
                    var deviceTypesInZone = {
                        LIGHT: false,
                        LIGHTpb: [],
                        SHADOW: false,
                        SHADOWpb: [],
                        HEATING: false,
                        HEATINGpb: [],
                        COOLING: false,
                        COOLINGpb: [],
                        VENTILATION: false,
                        VENTILATIONpb: [],
                        RECIRCULATION: false,
                        RECIRCULATIONpb: [],
                        AUDIO: false,
                        AUDIOpb: [],
                        VIDEO: false,
                        VIDEOpb: [],
                        WHITE: false,
                        WHITEpb: []
                    };
                    var addZone = false;
                    devStore.each(function (device) {
                        var pbConfig;
                        if (device.get('zoneId') === record.get('zoneId')) {
                            if (!device.get('isPresent')) {
                                return;
                            }

                            if (dSS.util.getJokerGroup(device.get('groups')) === dSS.config.groups.APT_VENTILATION) {
                                addApartmentVentilation = true;
                            }

                            var buttonID = device.get('buttonID');
                            if (
                                buttonID !== 0xff &&
                                (device.get('productRevision') >= dSS.config.PushbuttonMinFW || device.get('isVdcDevice'))
                            ) {
                                pbConfig = (buttonID & 0x0000000f);
                            }

                            var fid = dSS.util.parseFunctionID(device.get('functionID'));
                            var joker;

                            switch (fid[0]) {
                                case dSS.config.deviceClass.GE:
                                    deviceTypesInZone.LIGHT = true;
                                    if (pbConfig != null) {
                                        deviceTypesInZone.LIGHTpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                case dSS.config.deviceClass.GR:
                                    if (device.get('dSUIDIndex') === 0) { // only first dsuid of GR-KL300 is shade (second one, if exists, is a sensor)
                                        deviceTypesInZone.SHADOW = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.SHADOWpb.push(pbConfig);
                                        }
                                        addZone = true;
                                    }
                                    break;
                                case dSS.config.deviceClass.BL:
                                    var groupMember = dSS.util.getHeatingGroup(device.get('groups'));

                                    if (groupMember === dSS.config.groups.COOLING) {
                                        deviceTypesInZone.COOLING = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.COOLINGpb.push(pbConfig);
                                        }
                                        addZone = true;
                                        break;
                                    }

                                    if (groupMember === dSS.config.groups.VENTILATION) {
                                        deviceTypesInZone.VENTILATION = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.VENTILATIONpb.push(pbConfig);
                                        }
                                        addZone = true;
                                        break;
                                    }

                                    if (groupMember === dSS.config.groups.RECIRCULATION) {
                                        deviceTypesInZone.RECIRCULATION = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.RECIRCULATIONpb.push(pbConfig);
                                        }
                                        addZone = true;
                                        break;
                                    }

                                    if (groupMember === dSS.config.groups.HEATING) {
                                        deviceTypesInZone.HEATING = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.HEATINGpb.push(pbConfig);
                                        }
                                        addZone = true;
                                        break;
                                    }
                                    break;
                                case dSS.config.deviceClass.TK:
                                    deviceTypesInZone.AUDIO = true;
                                    if (pbConfig != null) {
                                        deviceTypesInZone.AUDIOpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                case dSS.config.deviceClass.MG:
                                    deviceTypesInZone.VIDEO = true;
                                    if (pbConfig != null) {
                                        deviceTypesInZone.VIDEOpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                case dSS.config.deviceClass.SW:
                                    deviceTypesInZone.JOKER = true;
                                    joker = dSS.util.getJokerGroup(
                                        device.get('groups')
                                    );
                                    if (joker === dSS.config.groups.LIGHT) {
                                        deviceTypesInZone.LIGHT = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.LIGHTpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.SHADOW) {
                                        deviceTypesInZone.SHADOW = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.SHADOWpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.HEATING) {
                                        deviceTypesInZone.HEATING = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.HEATINGpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.AUDIO) {
                                        deviceTypesInZone.AUDIO = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.AUDIOpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.VIDEO) {
                                        deviceTypesInZone.VIDEO = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.VIDEOpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.COOLING) {
                                        deviceTypesInZone.COOLING = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.COOLINGpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.VENTILATION) {
                                        deviceTypesInZone.VENTILATION = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.VENTILATIONpb.push(pbConfig);
                                        }
                                    } else if (joker === dSS.config.groups.RECIRCULATION) {
                                        deviceTypesInZone.RECIRCULATION = true;
                                        if (pbConfig != null) {
                                            deviceTypesInZone.RECIRCULATIONpb.push(pbConfig);
                                        }
                                    }
                                    addZone = true;
                                    break;
                                case dSS.config.deviceClass.WE:
                                    deviceTypesInZone.WHITE = true;
                                    if (pbConfig != null) {
                                        deviceTypesInZone.WHITEpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                default:
                                    break;
                            }
                        }
                    });
                    if (addZone) {
                        nodes.push({
                            text: record.get('name'),
                            id: record.get('zoneId'),
                            leaf: false,
                            singleClickExpand: this.annoyingMode,
                            children: this.generateSubNodes(record, deviceTypesInZone, record.get('clusterLocks'))
                        });
                    }
                }
                return true;
            }, this
        );

        // there is at least one device in apartment ventilation group
        // available, add the respective menu
        if (addApartmentVentilation && dSS.data.Loader.AVscenes.length) {
            nodes.push(this.generateSeparatorNode());
            nodes.push({
                text: dSS._('Apartment ventilation'),
                id: 0,
                leaf: false,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL],
                singleClickExpand: this.annoyingMode,
                children: this.generateApartmentVentilationNodes(
                    dSS.data.Loader.AVscenes
                )
            });
        }

        var root = new Ext.tree.AsyncTreeNode({
            cls: 'dsClsScenesPanel_TreePanel_Root',

            children: nodes,
            id: 'root',
            singleClickExpand: this.annoyingMode,
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length) {
            var tp = this.treePanel;
            panel.selectPath(selectedPath, null, function (result) {        // , node
                // if a node is gone, find the parend and try to select the
                // first child but othewise stay on the same level in the tree
                if (result === false) {
                    var selected = false;
                    var nodes = selectedPath.split('/');
                    if (nodes.length >= 2) {
                        var parentId = nodes[nodes.length - 2];
                        var parentNode = tp.getNodeById(parentId);
                        if (parentNode && parentNode.firstChild) {
                            parentNode.firstChild.select();
                            selected = true;
                        }
                    }
                    if (!selected) {
                        if (root.firstChild) {
                            root.firstChild.select();
                            selected = true;
                        }
                    }

                    if (!selected) {
                        if (this.apartmentTreePanel.getRootNode() && this.apartmentTreePanel.getRootNode().firstChild) {
                            this.apartmentTreePanel.getRootNode().firstChild.select();
                            selected = true;
                        }
                    }
                }
            });
        } else if (root.firstChild) {
            root.firstChild.select();
        } else if (this.apartmentTreePanel.getRootNode() && this.apartmentTreePanel.getRootNode().firstChild) {
            this.apartmentTreePanel.getRootNode().firstChild.select();
        }

        if (selModel.getSelectedNode() != null) {
            this.handleNodeClick(selModel.getSelectedNode());
        } else {
            selModel = this.apartmentTreePanel.getSelectionModel();
            if (selModel.getSelectedNode() != null) {
                this.handleNodeClick(selModel.getSelectedNode());
            }
        }

        if (this.activeNode) {
            var activeNode = panel.getNodeById(this.activeNode.id);
            if (activeNode == null) {
                activeNode = this.apartmentTreePanel.getNodeById(this.activeNode.id);
            }
            this.activeNode = activeNode;
            if (this.activeNode) {
                this.activeNode.setCls('active-scene-node');
            }
        }

        this.updateApartmentSceneName(dSS.config.scenes.ALARM1);
        this.updateApartmentSceneName(dSS.config.scenes.ALARM2);
        this.updateApartmentSceneName(dSS.config.scenes.ALARM3);
        this.updateApartmentSceneName(dSS.config.scenes.ALARM4);

        this.dirtyTree = false;
        this.fireEvent('treeredrawcomplte');
    },

    handleNodeClick: function (node) {
        if (!this.isVisible()) {
            return;
        }

        if (node == null) {
            return;
        }

        if (node.isLeaf() && !node.attributes.nocontextmenu) {
            this.menuPanel.getTopToolbar().get('callSceneButton').enable();
            if (node.attributes.undo) {
                this.menuPanel.getTopToolbar().get('undoSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
            }
        } else {
            this.menuPanel.getTopToolbar().get('callSceneButton').disable();
            this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        var ids;
        if (node.attributes.rename || path.length === 4) {
            ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId != null) {
                this.menuPanel.getTopToolbar().get('renameSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            }
            dSS.MessageBus.fireEvent('sceneselected', node.id.split('-')[2]);
        } else {
            this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            dSS.MessageBus.fireEvent('sceneselected', 0);
        }

        if (path.length > 2 && !node.attributes.noreadareaconfig) {
            ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId !== null) {
                this.menuPanel.getTopToolbar().get('readAreaConfigButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('readAreaConfigButton').disable();
            }
        } else {
            this.menuPanel.getTopToolbar().get('readAreaConfigButton').disable();
        }

        if (path.length < 2) {
            return;
        }

        var zoneId = parseInt(path[1], 10);
        if (zoneId === dSS.config.zones.BROADCAST) {
            this.controlPanel.hide();
            this.contentPanel.devicePanel.reconfigureColumns('apartment');
            this.doLayout();
        } else {
            this.controlPanel.show();
            this.contentPanel.devicePanel.reconfigureColumns('zone');
            this.doLayout();
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        var sm = this.contentPanel.devicePanel.getSelectionModel();
        var rec = sm.getSelected();
        var selectedDSUID = '';
        if (rec) {
            selectedDSUID = rec.get('dsuid');
        }
        store.filterBy(function (record) {
            if (!record.get('isPresent')) {
                return false;
            }

            if (zoneId === dSS.config.zones.BROADCAST) {
                return true;
            }

            if (record.get('zoneId') === zoneId) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));
                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === dSS.config.deviceClass.BL) {
                            return dSS.util.getHeatingGroup(record.get('groups')) === group;
                        }

                        if (fid[0] == dSS.config.deviceClass.WE &&
                            group == dSS.config.groups.SINGLEDEVICES) {
                            // its a single-group device
                            return true;
                        }

                        if (fid[0] === group) {
                            // Patch for DSD-10063 - GR-KL300 second dsuid (sensor) has wrong functionID, suggesting it beloongs to group 2.
                            // It can't be fixed now in TB, as it's already in production.
                            var isGrKl300TypeDevice = dSS.util.isGrKl300TypeDevice(record);
                            if (isGrKl300TypeDevice && isGrKl300TypeDevice.sensor) {
                                return false;
                            }
                            return true;
                        }

                        if (fid[0] === dSS.config.deviceClass.SW &&
                            dSS.util.getJokerGroup(record.get('groups')) === group) {
                            return true;
                        }

                        return false;
                    }
                } else {
                    return true;
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);

                if (isNaN(scene)) {
                    return false;
                }
            } else if (isNaN(zoneId)) { // apartment ventilation
                if (dSS.util.getJokerGroup(record.get('groups')) === dSS.config.groups.APT_VENTILATION) {
                    return true;
                }
            }

            return false;
        });

        var devices = store.getRange();
        if (zoneId !== dSS.config.zones.BROADCAST) {
            if (devices.length) {
                this.controlPanel.settings.loadDevices(devices);
            }
            if (node.isLeaf() && !node.attributes.lockedScene) {
                this.controlPanel.enable();
                ids = this.getZoneSceneGroup(node.id);
                var data = this.getNodeDisplayData(ids.zoneId, ids.groupId, ids.sceneNumber);
                this.dirtyCount = 0;
                this.dirtyDevices = {};
                this.setSaveText(data.name, this.dirtyCount);
                this.discardButton.disable();
            } else {
                this.controlPanel.disable();
            }
        }
        if (node.isLeaf() && devices.length) {
            sm.selectRow(0, false, false);
            if (devices[0].get('dsuid') !== selectedDSUID || this.activeNode) {
                rec = devices[0];
                this.contentPanel.devicePanel.handleDeviceRowSelect();
            } else {
                this.updateSaveSceneAndSettingsPanel(rec);
            }

            this.controlPanel.settings.setupButtons();
        } else {
            this.contentPanel.devicePanel.getSelectionModel().clearSelections();
            this.controlPanel.settings.clearDialog();
        }
    },

    updateSaveSceneAndSettingsPanel: function (record) {
        if (this.contentPanel.devicePanel.deviceValuesAndSceneConfigAvailable(record)) {
            this.controlPanel.settings.enable();
            this.controlPanel.saveScenePanel.enable();
        } else {
            this.controlPanel.settings.disable();
            this.controlPanel.saveScenePanel.disable();
        }
    },

    buildTopToolbar: function () {
        return [ {
            iconCls: 'editIcon',
            handler: this.renameSelectedScene.createDelegate(this, [ null ]),
            tooltip: dSS._('Rename selected scene'),
            id: 'renameSceneButton',
            disabled: true,
            scope: this
        },
        '-',
        {
            iconCls: 'callSceneIcon',
            handler: this.processSelectedScene.createDelegate(this, [ false ]),
            id: 'callSceneButton',
            tooltip: dSS._('Call selected scene'),
            disabled: true,
            scope: this
        }, {
            iconCls: 'undoSceneIcon',
            handler: this.processSelectedScene.createDelegate(this, [ true ]),
            id: 'undoSceneButton',
            tooltip: dSS._('Undo selected scene'),
            disabled: true,
            scope: this
        }, {
            iconCls: 'readAreaConfigIcon',
            handler: this.readAreaConfig.createDelegate(this),
            id: 'readAreaConfigButton',
            tooltip: dSS._('Read area configuration of devices ' +
                        'for selected zone and group'),
            disabled: true,
            scope: this
        },
        '->',
        {
            iconCls: 'showAllScenesIcon',
            handler: this.showAllScenes.createDelegate(this),
            tooltip: dSS._('Display all available scenes'),
            ref: 'showAllScenesButton',
            enableToggle: true,
            scope: this
        } ];
    },

    showAllScenes: function (button) {
        if (button.pressed) {
            this.disableSceneFilter = true;
        } else {
            this.disableSceneFilter = false;
        }
        // might get called from "save as window", sync button
        if (button.pressed !== this.menuPanel.getTopToolbar().showAllScenesButton.pessed) {
            this.menuPanel.getTopToolbar().showAllScenesButton.toggle(
                button.pressed, false
            );
        }
        this.markDirty();
    },

    getZoneSceneGroup: function (id) {
        var ret = {
            zoneId: null,
            sceneNumber: null,
            groupId: null
        };

        var ids = id.toString().split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    getSelectedScene: function () {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if (node == null || !node.isLeaf()) {
            throw dSS._('Can not identify scene: no scene selected');
        }

        var ids = this.getZoneSceneGroup(node.id);
        if (ids.zoneId == null || ids.sceneNumber == null) {
            throw dSS._('Can not identify scene: invalid data');
        }

        return ids.sceneNumber;
    },

    processSelectedScene: function (undo) {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if (node == null || !node.isLeaf()) {
            if (undo) {
                Ext.Msg.alert(dSS._('Error'), dSS._('No scene selected, please select the scene that you would like to undo'));
            } else {
                Ext.Msg.alert(dSS._('Error'), dSS._('No scene selected, please select the scene that you would like to call'));
            }
            return;
        }

        this.processScene(node, undo);
    },

    readAreaConfig: function () {
        if (!dSA.module) {
            Ext.Msg.alert(
                dSS._('Notice'),
                dSS._('This feature requires presence of the digitalSTROM Assistant, the setup is only available on the dSS11 hardware')
            );
            return;
        }

        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            Ext.Msg.alert(dSS._('Error'), dSS._('Nothing selected'));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);
        if (ids.zoneId == null || ids.groupId == null) {
            return;
        }

        dSA.Reactor.fireEvent('readareaconfig', ids.zoneId, ids.groupId, dSS.data.Loader.getDeviceStore());
    },

    processScene: function (node, undo) {
        if (node == null || !node.isLeaf()) {
            return;
        }

        if (undo && !node.attributes.undo) {
            Ext.Msg.alert(dSS._('Error'),
                dSS._('Undo is not available for this scene'));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);
        if (ids.zoneId == null || ids.sceneNumber == null) {
            return;
        }

        if (this.activeNode) {
            this.activeNode.setCls();
        }

        node.setCls('active-scene-node');
        this.activeNode = node;
        this.discardButton.disable();
        this.dirtyCount = 0;
        this.dirtyDevices = {};
        var data = this.getNodeDisplayData(ids.zoneId, ids.groupId, ids.sceneNumber);
        this.setSaveText(data.name, this.dirtyCount);
        var overrideZoneID = ids.zoneId;
        if (isNaN(ids.zoneId)) {
            // apartment ventilation
            overrideZoneID = 0;
        }
        if (undo) {
            dSS.util.undoSceneOnZone(overrideZoneID, ids.sceneNumber);
        } else {
            dSS.util.callSceneOnZone(overrideZoneID, ids.sceneNumber, ids.groupId, true);
        }
        if (ids.zoneId !== dSS.config.zones.BROADCAST) {
            var sm = this.contentPanel.devicePanel.getSelectionModel();
            if (sm.hasSelection()) {
                this.controlPanel.settings.updateView(false);
            } else {
                this.controlPanbel.settings.clearDialog();
            }
        }
    },

    getSelectedNode: function () {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }
        return node;
    },

    saveSelectedScene: function () {
        var node = this.getSelectedNode();
        if (node == null || !node.isLeaf()) {
            Ext.Msg.alert(
                dSS._('No scene selected'),
                dSS._('No scene selected, please select the scene that you would like to save')
            );
            return;
        }

        this.saveScene(node);
    },

    saveScene: function (node) {
        if (node == null || !node.isLeaf()) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if (ids.zoneId == null || ids.sceneNumber == null) {
            return;
        }

        if (isNaN(ids.zoneId)) {
            ids.zoneId = dSS.config.zones.BROADCAST;
        }

        if (ids.sceneNumber !== dSS.config.scenes.T0_S0) {
            if (this.activeNode != null && node !== this.activeNode) {
                this.warnSaveDifferentScene(node, this.activeNode);
                return;
            }
            if (this.activeNode) {
                this.activeNode.setCls();
            }
            node.setCls('active-scene-node');
            this.activeNode = node;
            var data = this.getNodeDisplayData(ids.zoneId, ids.groupId, ids.sceneNumber);
            this.dirtyCount = 0;
            this.dirtyDevices = {};
            this.setSaveText(data.name, this.dirtyCount);
            this.discardButton.disable();

            dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);

            if (ids.sceneNumber >= dSS.config.scenes.T1_S1 && ids.sceneNumber <= dSS.config.scenes.T4_S1) {
                var store = dSS.data.Loader.getDeviceStore();
                store.each(function (record) {
                    record.set('areas', []);
                });
            }
        } else {
            this.warnSaveScene0(node);
        }
    },

    warnSaveDifferentScene: function (node, activeNode) {
        var ids = this.getZoneSceneGroup(node.id);
        if (ids.zoneId == null || ids.sceneNumber == null) {
            return;
        }

        if (isNaN(ids.zoneId)) {
            ids.zoneId = dSS.config.zones.BROADCAST;
        }

        var aIds = this.getZoneSceneGroup(activeNode.id);
        if (aIds.zoneId == null || aIds.sceneNumber == null) {
            return;
        }

        var calledName = dSS._(dSS.config.sceneNames[aIds.sceneNumber]) +
            this.getSceneName(aIds.zoneId, aIds.groupId, aIds.sceneNumber);

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._('Save scene %1%2%3'), [ '&quot;', savingName, '&quot;' ]);

        var msg = dSS._('Called scene') + ': &quot;' + calledName + '&quot;<br/>' +
            dSS._('Saving scene') + ': &quot;' + savingName + '&quot;<br/><br/>' +
            dSS._('You are about to save a scene that is different from the called scene.') + '<br/>' +
            dSS._('Are you sure that you want to overwrite the scene?');

        Ext.Msg.confirm(
            title,
            msg,
            function (btn) {
                if (btn === 'yes') {
                    if (this.activeNode) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;
                    this.dirtyCount = 0;
                    this.dirtyDevices = {};
                    this.setSaveText(savingName, this.dirtyCount);
                    this.discardButton.disable();
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
                    if (ids.sceneNumber >= dSS.config.scenes.T1_S1 && ids.sceneNumber <= dSS.config.scenes.T4_S1) {
                        var store = dSS.data.Loader.getDeviceStore();
                        store.each(function (record) {
                            record.set('areas', []);
                        });
                    }
                }
            }, this
        );
    },

    warnSaveScene0: function (node) {
        var ids = this.getZoneSceneGroup(node.id);
        if (ids.zoneId == null || ids.sceneNumber == null) {
            return;
        }

        if (isNaN(ids.zoneId)) {
            ids.zoneId = dSS.config.zones.BROADCAST;
        }

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._('Save scene %1%2%3'), [ '&quot;', savingName, '&quot;' ]);

        var msg = dSS._('You are about to save scene 0 which represents the off scene. Note that you might not be able to ' +
            'switch off all devices in case some devices in the selected room are turned on right now.') + '<br/>' +
            dSS._('Are you sure you want to save the scene?');

        Ext.Msg.confirm(
            title,
            msg,
            function (btn) {
                if (btn === 'yes') {
                    if (this.activeNode) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;
                    this.dirtyCount = 0;
                    this.dirtyDevices = {};
                    this.setSaveText(savingName, this.dirtyCount);
                    this.discardButton.disable();
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
                }
            }, this
        );
    },

    renameSelectedScene: function (node) {
        var sel;

        if (node == null) {
            sel = this.treePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if (node == null || !node.isLeaf()) {
            Ext.Msg.alert(dSS._('Error'), dSS._('No scene selected, please select the scene that you want to rename'));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if (ids.zoneId == null || ids.groupId == null || ids.sceneNumber == null) {
            Ext.Msg.alert(dSS._('Error'), dSS._('Could not determine scene from node'));
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var recId = ids.zoneId + '-' + ids.groupId + '-' + ids.sceneNumber;

        var record = sceneStore.getById(recId);

        // this should never be the case, however if by any chance we missed
        // something I prefer having a messagebox here instead of some
        // random error
        if (record == null) {
            record = new sceneStore.recordType({
                name: '',
                scene: ids.sceneNumber,
                group: ids.groupId,
                zone: ids.zoneId
            }, recId);
            sceneStore.add([record]);
        }

        Ext.Msg.prompt(
            dSS._('Rename scene'),
            dSS._('New name for scene'),
            function (btn, text) {
                if (btn === 'ok' && text !== record.get('name')) {
                    Ext.Ajax.request({
                        url: '/json/zone/sceneSetName',
                        disableCaching: true,
                        method: 'GET',
                        params: {
                            id: ids.zoneId,
                            groupID: ids.groupId,
                            sceneNumber: ids.sceneNumber,
                            newName: text
                        },
                        success: function (result) {    // , request
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    var old = record.get('name');
                                    record.set('name', text);
                                    sceneStore.commitChanges();
                                    dSS.MessageBus.fireEvent('statusmessage',
                                        dSS.gt.strargs(dSS._('Scene %1 was renamed. New name: %2'),
                                            [ dSS.util.getName(old), text ]));
                                } else {
                                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not rename scene'));
                                }
                            } catch (err) {
                                Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not rename scene') + ': ' + err);
                            }
                        },
                        failure: function () {      // result, request
                            Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not rename scene'));
                        }
                    });
                }
            },
            this,
            false,
            record.get('name')
        );
    },

    getSelectedSceneList: function () {
        var node = this.getSelectedNode();
        if (!node || !node.isLeaf()) {
            // should never happen because the button is only shown when a
            // leaf node is selected
            return;
        }

        var sceneList = [];
        var scope = this;
        node.parentNode.eachChild(function (entry) {
            // don't draw separator nodes and also exclude currently selected
            // node
            if (!entry.isLeaf() || entry.attributes.isSeparator) {
                return true;
            }
            var ids = scope.getZoneSceneGroup(entry.id);
            var item = [
                ids.sceneNumber,
                ids.zoneId,
                ids.groupId,
                entry.attributes.iconCls,
                entry.text
            ];
            sceneList.push(item);
            return true;
        });
        return sceneList;
    },

    saveToAnotherScene: function () {
        var win = new dSS.SaveSceneDialogWindow({
            sceneList: this.getSelectedSceneList(),
            caller: this,
            sceneToggleState: this.disableSceneFilter,
            height: 170,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show(this);
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);
