Ext.namespace('dSS');

dSS.AddZoneWindow = Ext.extend(Ext.Window, {
    constructor: function(config)  {
        this.floorData = [];
        if (config.floorData !== undefined) {
            this.floorData = config.floorData;
        }

        this.floorStore = new Ext.data.ArrayStore({
            id: 0,
            fields:
            [ 'id', 'name', 'order' ],
            data: this.floorData
        });

        this.floorSelection = new Ext.form.ComboBox({
            cls: 'dsClsAddZoneWindow_FloorSelectionCombo',

            width: 205,
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:false,
            forceSelection: true,
            autoSelect: true,
            allowBlank: false,
            editable: false,
            mode: 'local',
            fieldLabel: dSS._("Create room on floor"),
            labelStyle: 'padding: 3px 0px 3px 6px;',
            store: this.floorStore,
            valueField: 'id',
            displayField: 'name'
        });

        this.name = new Ext.form.TextField({
            cls: 'dsClsAddZoneWindow_NameTextField',

            ref: 'name',
            width: 205,
            fieldLabel: dSS._("Name for new room"),
            enableKeyEvents: true,
            labelStyle: 'padding: 3px 0px 3px 6px;',
        });

        Ext.apply(this, {
            cls: 'dsClsAddZoneWindow',

            border: false,
            scope: this,
            padding: 10,
            width: 350,
            layout: 'form',
            bodyCfg: {
                cls: 'x-window-body x-window-mc'
            },
            title: dSS._("Create new room"),
            items:
            [
                {
                    xtype: 'box',
                    height: 10
                },
                this.name,
                this.floorSelection
            ],
            buttons:
            [
                {
                    text: dSS._("OK"),
                    handler: this.createZone.createDelegate(this)
                },
                {
                    text: dSS._("Cancel"),
                    handler: this.cancelZone.createDelegate(this)
                }
            ]
        });

        dSS.AddZoneWindow.superclass.constructor.apply(this, arguments);

        this.on('show', function() {
            this.floorSelection.setValue(dSS.config.floors.APARTMENT);
        }, this);

        this.name.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                this.createZone();
            }
        }, this);
    },
    createZone: function() {
        var zoneStore = dSS.data.Loader.getZoneStore();
        var floorID = this.floorSelection.getValue();
        var record = this.floorSelection.getStore().getById(floorID);
        var order = 0;
        if (record !== undefined) {
            order = record.get('order');
        }
        var name = this.name.getValue();
        var i;
        var win = this;
        for (i = 0; i <= zoneStore.data.length; i++) {
            if (zoneStore.findExact('zoneId', i) === - 1) {
                Ext.Ajax.request({
                    url: '/json/structure/addZone',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        zoneID: i,
                        floorID: floorID
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                               dSS.MessageBus.fireEvent('statusmessage',
                                 dSS.gt.strargs(dSS._("Zone %1 was created"), [i]));

                                Ext.Ajax.request({
                                    url: '/json/zone/setName',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        id: i,
                                        newName: name
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(result.responseText);
                                            if ((name === undefined) ||
                                                (name.length === 0)) {
                                                 name = dSS.gt.strargs(dSS._("Zone #%1"), [i]);
                                            }
                                            if (jsonData.ok) {
                                                zoneStore.loadData({
                                                    zones: [
                                                    {
                                                        zoneId: i,
                                                        name: name,
                                                        floorId: floorID,
                                                        alternativeSort: order
                                                    }]
                                                },
                                                true);
                                                win.close();
                                            } else {
                                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + jsonData.message);
                                            }
                                        }
                                        catch(err) {
                                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                                    }
                                });

                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                    }
                });
                return;
            }
        }
    },
    cancelZone: function() {
        this.close();
    }
});

dSS.ZoneView = Ext.extend(Ext.DataView, {
    constructor: function(config) {

        this.featureConfig = dSS.config.FeatureConfig.ZONE;
        this.lastSelectedZoneID = undefined;

        if (config.featureConfig != undefined) {
            this.featureConfig = config.featureConfig;
        }

        var unnamedDevicesTooltip = dSS._('Contains unnamed devices');

        var zoneTemplate = new Ext.XTemplate(
            '<tpl for=".">',
                '<tpl if="!isFloor">',
                    '<div class="zone-wrap {css}" id="zone-{zoneId}">',
                        '<tpl if="unnamedDevices">',
                            '<span title="' + unnamedDevicesTooltip + '">',
                                '<tpl if="((zoneId !== dSS.config.zones.BROADCAST) && (zoneId !== dSS.config.zones.NEW_DEVICES))">&nbsp;&nbsp;&nbsp;&nbsp</tpl>',
                                '{name} *',
                            '</span>',
                        '</tpl>',
                        '<tpl if="!unnamedDevices">',
                            '<span>',
                                '<tpl if="((zoneId !== dSS.config.zones.BROADCAST) && (zoneId !== dSS.config.zones.NEW_DEVICES))">&nbsp;&nbsp;&nbsp;&nbsp</tpl>',
                                '{name}',
                            '</span>',
                        '</tpl>',
                    '</div>',
                '</tpl>',
                '<tpl if="isFloor">',
                    '<div class="floor-background bold-text zone-wrap {css}" id="floor-{floorId}">',
                        '<tpl if="unnamedDevices">',
                            '<span title="' + unnamedDevicesTooltip + '">',
                                '{name} *',
                            '</span>',
                        '</tpl>',
                        '<tpl if="!unnamedDevices">',
                            '<span>',
                                '{name}',
                            '</span>',
                        '</tpl>',
                    '</div>',
                '</tpl>',
            '</tpl>',
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            cls: 'dsClsZoneView',

            store: dSS.data.Loader.getZoneStore(),
            tpl: zoneTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: dSS._("No rooms to display")
        });

        dSS.ZoneView.superclass.constructor.apply(this, arguments);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('selectionchange', function(dv, nodes) {
            if (nodes.length > 0) {
                var selectedZones = this.getSelectedRecords();
                if (selectedZones.length > 0) {
                    this.lastSelectedZoneID = selectedZones[0].get('zoneId');
                }
            }

            this.filterDevices();
        },
        this);

        dSS.MessageBus.on('structureloaded', function() {
            // #7453 can't say why "this.isVisible()" is not working properly,
            // same goes for the first owner container, so we have to go two
            // levels up to reach the panel that is actually the "tab" itself,
            // in this case it will be either the dsszonebrowser or the
            // dsssensorbrowser
            if (this.ownerCt.ownerCt) {
                if (!this.ownerCt.ownerCt.isVisible()) {
                    return;
                }
            }

            if (!this.ownerCt.isVisible()) {
                return;
            }

            if (!this.isVisible()) {
                return;
            }

            if (this.getStore().getCount() === 0) {
                this.getStore().clearFilter();
                this.filterDevices();
                return;
            }

            this.restoreSelection();
        },
        this);

        this.on('afterrender', function(v) {
            var zoneView = this;

            this.dragZone = new Ext.dd.DragZone(zoneView.getEl(), {
                cls: 'dsClsZoneView_DragZone',

                ddGroup: "zoneDeviceDD",
                containerScroll: true,
                getDragData: function(e) {
                    var sourceEl = e.getTarget(v.itemSelector, 10);
                    if (sourceEl) {
                        d = sourceEl.firstChild.cloneNode(true);
                        d.id = Ext.id();
                        d.innerText = v.getRecord(sourceEl).get('name');
                        return {
                            ddel: d,
                            sourceEl: sourceEl,
                            repairXY: Ext.fly(sourceEl).getXY(),
                            sourceStore: v.store,
                            draggedRecord: v.getRecord(sourceEl)
                        }
                    }
                },
                getRepairXY: function() {
                    return this.dragData.repairXY;
                },
                onBeforeDrag: function(data, evt) {
                    if ((data.draggedRecord.get('isFloor') === true) ||
                            ((data.draggedRecord.get('isFloor') === false) &&
                                ((data.draggedRecord.get('zoneId') === dSS.config.zones.BROADCAST) ||
                                 (data.draggedRecord.get('zoneId') === dSS.config.zones.NEW_DEVICES)))) {
                        return false;
                    }

                    return true;
                }
            });

            this.dropZone = new Ext.dd.DropZone(zoneView.getEl(), {
                cls: 'dsClsZoneView_DropZone',

                ddGroup: "zoneDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(zoneView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, dd, e, data) {
                    var dropRecord = zoneView.getRecord(target);
                    // a zone is being dragged
                    if (data.draggedRecord !== undefined) {
                        if ((dropRecord.get('isFloor') === false) ||
                            (dropRecord.get('floorId') === data.draggedRecord.get('floorId'))) {
                             return Ext.dd.DropZone.prototype.dropNotAllowed;
                        }
                    } else { // a device is being dragged
                        if ((dropRecord.get('isFloor') === true) ||
                            (dropRecord.get('zoneId') === dSS.config.zones.BROADCAST)) {
                            return Ext.dd.DropZone.prototype.dropNotAllowed;
                        }
                     }
                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);

                    // zone is being dropped
                    if (record.get('isFloor') === true) {
                        // zone selection supports only single select,
                        // so there will only be one zone there
                        var droppedZone = data.draggedRecord;
                        if (!droppedZone) {
                            return;
                        }

                        if (droppedZone.get('floorId') === record.get('floorId')) {
                            return;
                        }

                        dSS.util.genericRequest(
                            {
                                url: '/json/structure/floorAddZone',
                                params:
                                {
                                    floorID: record.get('floorId'),
                                    zoneID: droppedZone.get('zoneId')
                                },
                                failureMsg: dSS._("Could not add zone to floor"),
                                scope: this
                            },
                            function(ok) {
                                if (ok === true) {
                                    var zonerec = dSS.util.getZoneById(droppedZone.get('zoneId'));
                                    zonerec.set('floorId', record.get('floorId'));
                                    zonerec.set('alternativeSort', record.get('alternativeSort'));
                                    zonerec.commit();
                                    dSS.data.Loader.getZoneStore().fireEvent('resort');
                                }
                            }
                        );
                    } else { // device is being dropped
                        if (record.get('zoneId') === 0) {
                            return false;
                        }

                        Ext.each(data.selections, function(device) {
                            var currentDevice = device;

                            if (currentDevice.get('isPresent') === false) {
                                return;
                            }

                            if (currentDevice.get('zoneId') === record.get('zoneId')) {
                                dSS.MessageBus.fireEvent('statusmessage',
                                    dSS.gt.strargs(
                                        dSS._("Device %1 could not be moved into " +
                                              "zone %2: device is already in this" +
                                              " zone"),
                                        [
                                            '&quot;' +
                                            dSS.util.getName(
                                                currentDevice.get('name')) +
                                            '&quot; (' +
                                             currentDevice.get('DisplayID') +
                                             ')',
                                             '&quot;' + record.get('name') +
                                             '&quot;'
                                        ]
                                    )
                                );
                                return;
                            }

                            Ext.Ajax.request({
                                url: '/json/structure/zoneAddDevice',
                                disableCaching: true,
                                method: "GET",
                                scope: zoneView,
                                params: {
                                    dsuid: currentDevice.get('dsuid'),
                                    zone: record.get('zoneId')
                                },
                                success: function(result, request) {
                                    try {
                                        var jsonData = Ext.util.JSON.decode(result.responseText);
                                        if (jsonData.ok) {
                                            this.moveDevices(jsonData.result,
                                                record,
                                                currentDevice.get('zoneId'));
                                        } else {
                                            Ext.Msg.alert(dSS._("Error"),
                                                dSS._("Device") +
                                                ' ' + currentDevice.get('DisplayID') +
                                                ' ' + dSS._("could not be moved") +
                                                ': ' + dSS._(jsonData.message));
                                        }
                                        dSS.data.Loader.getDeviceStore().getSensorAssignments();
                                    } catch(err) {
                                        Ext.Msg.alert(
                                            dSS._("Error"), dSS._("Device") + ' ' +
                                            currentDevice.get('DisplayID') + ' ' +
                                            dSS._("could not be moved") +
                                            ': ' + dSS._(err));
                                        dSS.MessageBus.fireEvent('reloadapartment');
                                    }
                                },
                                failure: function(result, request) {
                                    Ext.Msg.alert(
                                        dSS._("Error"), dSS._("Device") +  ' ' +
                                        currentDevice.get('DisplayID') + ' ' +
                                        dSS._("could not be moved"));
                                    dSS.MessageBus.fireEvent('reloadapartment');
                                }
                            });
                        });
                        return true;
                    }
                }
            });
        },
        this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();

            if (this.featureConfig !== dSS.config.FeatureConfig.ZONE) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({
                    cls: 'dsClsZoneView_ContextMenu'
                });
            } else {
                this.contextMenu.removeAll();
            }
            this.select(index);

            var record = this.getStore().getAt(index);

            var menuItem;

            if (record.get('isFloor') !== true) {
                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_IdentificationBlinkItem',

                    text: dSS._("Identification Blink"),
                    iconCls: 'blink',
                    handler: this.blinkZone.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_CreateNewRoomItem',

                    text: dSS._("Create new room"),
                    iconCls: 'addZoneIcon',
                    handler: this.createNewZone.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);

                var disable = false;
                if ((record.get('zoneId') === dSS.config.zones.BROADCAST) ||
                    (record.get('zoneId') === dSS.config.zones.NEW_DEVICES)) {
                    disable = true;
                }

                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_RenameRoomItem',

                    text: dSS._("Rename room"),
                    iconCls: 'renameZoneIcon',
                    disabled: disable,
                    handler: this.renameZone.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_DeleteRoomItem',

                    text: dSS._("Delete room"),
                    iconCls: 'removeZoneIcon',
                    disabled: disable,
                    handler: this.removeZone.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);
            } else {
                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_CreateNewFloorItem',

                    text: dSS._("Create new floor"),
                    iconCls: 'addFloorIcon',
                    handler: this.createNewFloor.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_RenameFloorItem',

                    text: dSS._("Rename floor"),
                    iconCls: 'renameZoneIcon',
                    handler: this.renameZone.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);

                var disable = true;
                if (record.get('floorId') > dSS.config.floors.APARTMENT) {
                    disable = false;
                }
                menuItem = new Ext.menu.Item({
                    cls: 'dsClsZoneView_ContextMenu_DeleteFloorItem',

                    text: dSS._("Delete floor"),
                    iconCls: 'removeFloorIcon',
                    disabled: disable,
                    handler: this.removeFloor.createDelegate(this, index, true)
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    moveDevices: function(json, newZone, oldZoneId) {
        if (json.movedDevices && json.movedDevices.length > 0) {
            var store = dSS.data.Loader.getDeviceStore();
            for (var d = 0; d < json.movedDevices.length; d++) {
                var i = store.findExact('dsuid', json.movedDevices[d].dSUID);
                if (i < 0) {
                    continue;
                }
                var device = store.getAt(i);
                device.set('zoneId', newZone.get('zoneId'));
                device.set('zoneName', newZone.get('name'));
                device.set('assignedSensorTypes', []);
                device.commit();
                dSS.data.Loader.getZoneStore().fireEvent('zonedevnamechanged',
                                                         device);
                dSS.MessageBus.fireEvent('statusmessage',
                   dSS.gt.strargs(
                        dSS._("Device %1 was moved " +
                             "into zone %2"),
                        [
                            '&quot;' + dSS.util.getName(device.get('name')) +
                            '&quot; (' + device.get('DisplayID') +
                            ')',
                            '&quot;' + newZone.get('name') + '&quot;'
                        ]
                    )
                );
            }

            this.filterDevices();
            dSS.MessageBus.fireEvent('scenesmarkdirty');
            dSS.data.Loader.getZoneStore().fireEvent('zonedeviceleft',
                                                     oldZoneId);
        }
    },

    restoreSelection: function() {
        // no action if there is a valid selection
        var selectedZones = this.getSelectedRecords();
        if (selectedZones.length > 0) {
            this.filterDevices();
            return;
        }

        if (this.lastSelectedZoneID !== undefined) {
            var rec = dSS.util.getZoneById(this.lastSelectedZoneID);
            if (rec !== undefined) {
                this.select(rec);
            }
        } else {
            this.select(dSS.util.getZoneById(0));
        }
    },
    filterDevices: function() {
        var selectedZone;
        var selectedZones = this.getSelectedRecords(); // single select
        if (selectedZones.length > 0) {
            selectedZone = selectedZones[0];
        }
        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            dSS.MessageBus.fireEvent('filter_devices_zone', selectedZone);
        } else if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
            dSS.MessageBus.fireEvent('filter_devices_sensor', selectedZone);
        }
    },

    removeZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        if(record.zoneId === dSS.config.zones.NEW_DEVICES) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Removing 'New Devices' room is not allowed"));
            return;
        }

        var deviceStore = dSS.data.Loader.getDeviceStore();

        if (deviceStore.getRange().length > 0) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Can not delete a non empty room"));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeZone',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                zoneID: record.get('zoneId')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                          dSS.gt.strargs(dSS._('Zone "%1" was removed'),
                             [record.get('name')]));
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove room") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove room") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove room"));
            }
        });
    },

    blinkZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        Ext.Ajax.request({
            url: '/json/zone/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                id: record.get('zoneId')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                    } else {
                        dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Zone \"%1\" is blinking now"),
                            [
                                record.get('name')
                            ]));
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
            }
        });
    },
    renameZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        if (record.get('isFloor') !== true) {
            if (record.get('zoneId') === dSS.config.zones.BROADCAST) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Renaming 'All devices' entry is not allowed"));
                return;
            }
            if (record.get('zoneId') === dSS.config.zones.NEW_DEVICES) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Renaming 'New devices' entry is not allowed"));
                return;
            }
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: this,
            title: record.get('isFloor') === true ? dSS._("Rename floor") : dSS._("Rename room"),
            jumpMode: true,
            featureConfig: dSS.config.FeatureConfig.ZONE,
            callerScope: this,
            width: 350,
            dialogType: dSS.config.dialogTypes.RENAME_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    createNewZone: function() {
        var zoneStore = this.getStore();
        var floorData = [];
        zoneStore.each(function(record) {
            if (record.get('isFloor') === true) {
                floorData.push([ record.get('floorId'),
                                 record.get('name'),
                                 record.get('alternativeSort') ]);
            }
        });

        var win = new dSS.AddZoneWindow({
            floorData: floorData
        });
        win.show();
    },
    removeFloor: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No floor selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        if (record.get('isFloor') !== true) {
            return;
        }

        var floorId = record.get('floorId');

        if (floorId === dSS.config.floors.APARTMENT) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Removing 'Apartment' floor is not allowed"));
            return;
        }

        var zoneStore = this.getStore();

        var empty = true;
        zoneStore.each(function(record) {
            if ((record.get('isFloor') !== true) &&
                (record.get('floorId') === floorId)) {
                empty = false;
                return false;
            }
        });

        if (empty !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Can not delete a non empty floor, zones must be removed first"));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeFloor',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                floorID: record.get('floorId')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                          dSS.gt.strargs(dSS._("Floor \"%1\" was removed"),
                             [record.get('name')]));
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove floor") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove floor") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove floor"));
            }
        });
    },
    createNewFloor: function() {
        var zoneStore = this.getStore();
        Ext.Msg.prompt(dSS._("Create new floor"), dSS._("Name for new floor"), function(btn, text) {
            text = text.trim();
            if (btn === 'ok') {
                if (text.length == 0) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Creating floor without a name is not allowed!"));
                    return;
                }

                Ext.Ajax.request({
                    url: '/json/structure/addFloor',
                    disableCaching: true,
                    method: "GET",
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                dSS.MessageBus.fireEvent('statusmessage',
                                                dSS._("New floor was created"));

                                var floorID = jsonData.result.floorID;
                                var orderID = jsonData.result.orderID;

                                Ext.Ajax.request({
                                    url: '/json/structure/setFloorName',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        floorID: floorID,
                                        name: text
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(result.responseText);
                                            if ((text === undefined) ||
                                                (text.length === 0)) {
                                                text = dSS.gt.strargs(dSS._("Floor #%1"), [floorID]);
                                            }
                                            if (jsonData.ok) {
                                                zoneStore.loadData({
                                                    zones: [
                                                    {
                                                        floorId: floorID,
                                                        name: text,
                                                        isFloor: true,
                                                        alternativeSort: orderID
                                                    }]
                                                },
                                                true);
                                            } else {
                                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create floor") + ": " + jsonData.message);
                                            }
                                        }
                                        catch(err) {
                                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create floor") + ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create floor"));
                                    }
                                });

                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create floor"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create floor"));
                    }
                });
                return;
            }
        });
    }

});

Ext.reg('dsszoneview', dSS.ZoneView);
