Ext.namespace('dSS', 'dSS.data');

dSS.data.ZoneStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for zone records
        var zoneRecord = Ext.data.Record.create([
        {
            name: 'name'
        },
        {
            name: 'zoneId'
        },
        {
            name: 'primary',
            defaultValue: false
        },
        {
            name: 'unnamedDevices',
            defaultValue: false
        },
        {
            name: 'alternativeSort',
            defaultValue: 0
        },
        {
            name: 'sensors',
            defaultValue: []
        },
        {
            name: 'clusterLocks',
            defaultValue: []
        },
        {
            name: 'lockedApartmentScenes',
            defaultValue: []
        },
        {
            name: 'activeBasicScenes'
        },
        {
            name: 'floorId'
        },
        {
            name: 'isFloor',
            defaultValue: false
        }
        ]);

        this.addEvents('zonedevnamechanged', 'zonedeviceleft', 'resort');

        // a json reader to read the zone data
        var zoneReader = new Ext.data.JsonReader({
            root: 'zones'
        },
        zoneRecord);

        Ext.apply(this, {
            reader: zoneReader,
            sortInfo: {
                    field: 'alternativeSort',
                    direction: 'ASC'
                }
        });
        dSS.data.ZoneStore.superclass.constructor.call(this, arguments);

        this.on('zonedeviceleft', function(zoneId) {
            if (zoneId === 0) {
                return;
            }

            var zone = dSS.util.getZoneById(zoneId);
            if (zone === undefined) {
                return;
            }

            if (zone.get('isFloor') === true) {
                return;
            }

            this.checkUnnamed(zone);
        },
        this);

        this.on('zonedevnamechanged', function(device) {
            var name = device.get('name');
            var zero = dSS.util.getZoneById(0);
            var zone = dSS.util.getZoneById(device.get('zoneId'));
            if (zone === undefined) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Could not find zone" +
                              " with id") + device.get('zoneId'));
                return;
            }

            if ((zero !== undefined) &&
                (zero.get('unnamedDevices') === false) &&
                ((name === undefined) || (name.length === 0))) {
                zero.set('unnamedDevices', true);
                zero.commit();
            }

            // if all devices were named and one becomes unnamed we do not
            // need to cycle through all devies in zone but can tag the
            // zone right away
            if ((zone.get('unnamedDevices') === false) &&
                ((name === undefined) || (name.length === 0))) {
                zone.set('unnamedDevices', true);
                zone.commit();
                return;
            }

            this.checkUnnamed(zone, zero);
        },
        this);

        this.on('load', this.resort.createDelegate(this), this);
        this.on('resort', this.resort.createDelegate(this), this);
    },

    resort: function() {
        this.sort([{ field: 'alternativeSort', direction: 'ASC' },
                   { field: 'isFloor', direction: 'DESC' },
                   { field: 'name', direction: 'ASC' } ]);
    },
    checkUnnamed: function(zone, zero) {
        if (zone.get('isFloor') === true) {
            return;
        }

        var unnamed = false;
        var records = dSS.data.Loader.getDeviceStore().queryBy(function(rec) {
                    if (rec.get('zoneId') === zone.get('zoneId')) {
                        return true;
                    }
                    return false;
                });
        records.each(
            function(r) {
                if ((r.get('name') === undefined) ||
                    (r.get('name').length === 0)) {
                    unnamed = true;
                }
            });

        if (unnamed !== zone.get('unnamedDevices')) {
            zone.set('unnamedDevices', unnamed);
            zone.commit();
        }

        if (zero !== undefined) {
            records = this.getRange(0);
            unnamed = false;
            var i;
            for (i = 0; i < records.length; i++) {
                if ((records[i].get('zoneId') !== 0) &&
                     (records[i].get('unnamedDevices') === true)) {
                    unnamed = true;
                    break;
                }
            }

            if (unnamed !== zero.get('unnamedDevices')) {
                zero.set('unnamedDevices', unnamed);
                zero.commit();
            }
        }
    }
});

