/* globals Ext, dSS */

/* Turn-on delay, turn-off delay inputs */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.AKMDelayConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function (config) {
        this.device = config.device;
        this.matrix = config.matrix;

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.dSUID = this.device.get('dsuid');
        this.group = config.color;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.onDelay = new Ext.Panel({
            cls: 'dsClsAKMDelayConfigForm_OnDelay',

            border: false,
            height: 35,
            layout: 'hbox',
            items: [
                {
                    xtype: 'label',
                    text: dSS._('Turn-on delay') + ':',
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsAKMDelayConfigForm_OnDelay_MinutesSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'minutes',
                    minValue: 0,
                    maxValue: 100,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowNegative: false,
                    allowBlank: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    cls: 'dsClsAKMDelayConfigForm_OnDelay_SecondsSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0,
                    maxValue: 59,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowNegative: false,
                    allowBlank: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._('mm:ss'),
                    style: { marginTop: '5px' }
                }

            ]
        });

        this.offDelay = new Ext.Panel({
            cls: 'dsClsAKMDelayConfigForm_OffDelay',

            border: false,
            height: 25,
            layout: 'hbox',
            items: [
                {
                    xtype: 'label',
                    text: dSS._('Turn-off delay') + ':',
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsAKMDelayConfigForm_OffDelay_MinutesSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'minutes',
                    minValue: 0,
                    maxValue: 100,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowNegative: false,
                    allowBlank: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    cls: 'dsClsAKMDelayConfigForm_OffDelay_SecondsSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0,
                    maxValue: 59,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowNegative: false,
                    allowBlank: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._('mm:ss'),
                    style: { marginTop: '5px' }
                }
            ]
        });

        Ext.apply(this, {
            cls: 'dsClsAKMDelayConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged', 'formretry', 'updateshadow', 'formsetdeviceconfig' ],
            padding: 5,
            scope: this,
            items: [
                this.onDelay,
                this.offDelay
            ]
        });

        dSS.devprops.AKMDelayConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.onDelay.minutes.on('valid', function () {
                this.fireEvent('formchanged');
            }, this);

            this.onDelay.seconds.on('valid', function () {
                this.fireEvent('formchanged');
            }, this);

            this.onDelay.minutes.on('invalid', function () {
                this.onDelay.minutes.setValue(this.onDelay.minutes.startValue);
            }, this);

            this.onDelay.seconds.on('invalid', function () {
                this.onDelay.seconds.setValue(this.onDelay.seconds.startValue);
            }, this);

            this.offDelay.minutes.on('valid', function () {
                this.fireEvent('formchanged');
            }, this);

            this.offDelay.seconds.on('valid', function () {
                this.fireEvent('formchanged');
            }, this);


            this.offDelay.minutes.on('invalid', function () {
                this.offDelay.minutes.setValue(this.offDelay.minutes.startValue);
            }, this);

            this.offDelay.seconds.on('invalid', function () {
                this.offDelay.seconds.setValue(this.offDelay.seconds.startValue);
            }, this);

            this.onDelay.minutes.on('spin', function () {
                this.fireEvent('formchanged');
            }, this);

            this.onDelay.seconds.on('spin', function () {
                this.fireEvent('formchanged');
            }, this);

            this.offDelay.minutes.on('spin', function () {
                this.fireEvent('formchanged');
            }, this);

            this.offDelay.seconds.on('spin', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('beforedestroy', function () {
            this.closed = true;
            dSS.MessageBus.removeListener('inputtypechanged', this.updateVisibility, this);
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            if (data.loader !== dSS.devprops.Loaders.AKM_DELAY_CONFIG) {
                return;
            }

            if (!data.json.ok) {
                this.offDelay.minutes.markInvalid(dSS._(data.json.message));
                this.offDelay.seconds.markInvalid(dSS._(data.json.message));
                this.onDelay.minutes.markInvalid(dSS._(data.json.message));
                this.onDelay.seconds.markInvalid(dSS._(data.json.message));
                this.fireEvent('formloaded', true);
                return;
            }

            this.loadedOnDelay = data.json.result.ondelay;
            this.loadedOffDelay = data.json.result.offdelay;

            var seconds = this.loadedOnDelay / 1000;
            var minutes = seconds / 60;
            seconds %= 60;

            this.onDelay.minutes.setValue(Math.floor(minutes));
            this.onDelay.seconds.setValue(Math.floor(seconds));

            seconds = this.loadedOffDelay / 1000;
            minutes = seconds / 60;
            seconds %= 60;

            this.offDelay.minutes.setValue(Math.floor(minutes));
            this.offDelay.seconds.setValue(Math.floor(seconds));

            this.fireEvent('formloaded', true);
        }, this);

        this.on('submit', function () {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            if (!this.getForm().isValid()) {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Please correct values in the red marked fields and try again')
                );
                this.fireEvent('formretry');
                return;
            }

            var minutes = parseInt(this.onDelay.minutes.getValue(), 10);
            var seconds = parseInt(this.onDelay.seconds.getValue(), 10);
            var onMs = (seconds + (minutes * 60)) * 1000;

            minutes = parseInt(this.offDelay.minutes.getValue(), 10);
            seconds = parseInt(this.offDelay.seconds.getValue(), 10);
            var offMs = (seconds + (minutes * 60)) * 1000;

            if (this.loadedOnDelay === onMs && this.loadedOffDelay === offMs) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest.defer(2000, this, [
                {
                    url: '/json/device/setAKMInputTimeouts',
                    params:
                {
                    dsuid: this.dSUID,
                    ondelay: onMs,
                    offdelay: offMs
                },
                    failureMsg: dSS._('Could not save delay settings'),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        }, this);
    },
    updateVisibility: function (input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.show();
            this.enable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.hide();
            this.disable();
        }
        this.fireEvent('updateshadow');
    }
});

Ext.reg('dssakmdelayconfigform', dSS.devprops.AKMDelayConfigForm);
