/* globals Ext, dSS */

/* Input selection */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.AKMInputConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function (config) {
        this.device = config.device;
        this.matrix = config.matrix;

        this.forceSave = false;

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.inputSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._('Input'),
            labelStyle: 'margin-top: 5px',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_STANDARD,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_STANDARD])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_INVERTED,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_INVERTED])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_FALLINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_FALLINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_FALLINGEDGE])
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            cls: 'dsClsAKMInputConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                'formsetdeviceconfig' ],
            padding: 5,
            height: 35,
            scope: this,
            items: [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items: [
                        this.inputSelection
                    ]
                }
            ]
        });

        dSS.devprops.AKMInputConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.inputSelection.on('select', function (combo) {
                this.fireEvent('formchanged');
                var v = combo.getValue();
                dSS.MessageBus.fireEvent('akminputchanged', v);
            }, this);

            var val = this.inputSelection.getValue();
            if (val) {
                dSS.MessageBus.fireEvent('akminputchanged', val);
            }
        }, this);

        this.on('afterrender', function () {
            var val = this.inputSelection.getValue();
            if (val) {
                dSS.MessageBus.fireEvent('akminputchanged', val);
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
            dSS.MessageBus.removeListener('inputtypechanged',
                this.updateVisibility, this);
        }, this);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            if (data.loader !== dSS.devprops.Loaders.AKM_INPUT_PROPERTY) {
                return;
            }

            if (!data.json.ok) {
                this.inputSelection.markInvalid(dSS._(data.json.message));
                this.fireEvent('formloaded', true);
                return;
            }

            this.loadedValue = data.json.result.value;

            // UMR hack - set default
            if (!this.loadedValue) {
                this.loadedValue = dSS.config.ButtonInputMode.BUTTONINPUT_AKM_STANDARD;
            }

            this.inputSelection.setValue(this.loadedValue);
            this.fireEvent('formloaded', true);
        }, this);

        this.on('submit', function () {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            var value = this.inputSelection.getValue();
            var device = this.device;
            if (!this.forceSave && value === this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            if (this.matrix.setumr200config) {
                this.fireEvent('formsetdeviceconfig', { stateMode: dSS.config.ButtonInputModeV2Map[value] });
                return;
            }

            dSS.util.genericRequest.defer(1000, this, [
                {
                    url: '/json/device/setAKMInputProperty',
                    params: {
                        dsuid: device.get('dsuid'),
                        mode: value
                    },
                    failureMsg: dSS._('Could not save AKM input property'),
                    scope: this,
                    evt: 'formsubmitted'
                },
                function (ok, jsonData) {
                    if (!ok) {
                        return;
                    }
                    // device.commit() doesn't work as the device record, for some reason, is no longer connected with the store
                    // in some cases (e.g. GR-KL300)
                    const store = dSS.data.Loader.getDeviceStore();
                    const index = store.find('dsuid', device.get('dsuid'));
                    if (index !== -1) {
                        const deviceFromStore = store.getAt(index);
                        deviceFromStore.set('AKMInputProperty', value);
                        deviceFromStore.set('buttonInputMode', dSS.config.ButtonInputModeLookup[value]);
                        deviceFromStore.commit();
                    }
                    dSS.util.processDeviceActionResponse();
                }
            ]);
        }, this);
    },
    updateVisibility: function (input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.show();
            this.forceSave = true;
            this.enable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.hide();
            this.disable();
        }
        this.fireEvent('updateshadow');
    }
});

Ext.reg('dssakminputconfigform', dSS.devprops.AKMInputConfigForm);
