Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID

/* globals dSS, Ext */

dSS.devprops.GrKl300OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function (config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');
        // this.dialogType = config.type; // TODO: remove from where params are passed
        // this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.loadedValue = null;

        this.checkbox = new Ext.form.Checkbox({
            cls: 'dsClsOutputModeForm_OutputModeCheckbox',

            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._('Output enabled'),
            labelStyle: 'margin-top: 5px',
            value: false,
            markValid: function () {
                this.el.dom.style.outline = '';
            },
            markInvalid: function () {
                this.el.dom.style.outline = '1px solid #c30';
            },
            isValid: function () {
                return !this.el.dom.style.outline;
            }
        });

        Ext.apply(this, {
            cls: 'dsClsOutputModeForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged', 'cancelbuttonstate', 'updateshadow' ],
            padding: 5,
            scope: this,
            layout: 'form',
            items: [ this.checkbox ]
        });

        dSS.devprops.GrKl300OutputModeForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.on('change', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            var val;
            if (data.loader === dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                if (!data || !data.json || !data.json.ok) {
                    this.checkbox.markInvalid();
                } else {
                    val = data.json.result[this.dSUID][0].outputMode;
                    if (val === dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL_UNCALIBRATED) {
                        val = dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL;
                    }
                    this.loadedValue = val;
                    this.checkbox.setValue(val === dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL, true);
                }
                this.el.unmask();
                this.fireEvent('formloaded', true);

                var me = this;
                this.checkbox.on('check', function () {
                    console.log('check');
                    this.markValid();
                    me.fireEvent('formchanged');
                });
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
        }, this);

        this.on('submit', function () {
            this.saveSettings('formsubmitted');
        }, this);
    },

    readTurnTimeCalibrationStatus (dSUID, callback, eventName) {
        Ext.Ajax.request({
            url: '/json/device/getConfig',
            method: 'GET',
            disableCaching: true,
            scope: this,
            params: {
                dsuid: dSUID,
                class: 1,
                index: 92
            },
            success: callback,
            failure: function () {
                Ext.Msg.alert(dSS._('Error'), dSS._('Could not save output mode settings'));
                this.fireEvent(eventName);
            }
        });
    },
    setOutputMode (dSUID, value, eventName) {
        Ext.Ajax.request({
            url: '/json/device/setOutputMode',
            method: 'GET',
            disableCaching: true,
            scope: this,
            params: {
                dsuid: dSUID,
                modeID: value
            },
            success: function (result) {        // , request
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        throw 'error';
                    }

                    var store = dSS.data.Loader.getDeviceStore();
                    var record = store.getById(dSUID);
                    if (record) {
                        record.set('outputMode', parseInt(value, 10));
                        record.commit();
                    }

                    dSS.util.processDeviceActionResponse(jsonData.result);
                    this.fireEvent(eventName);
                } catch (err) {
                    Ext.Msg.alert(dSS._('Error'), dSS._('Could not save output mode settings'));
                    this.fireEvent(eventName);
                }
            },
            failure: function () {      // result, request
                Ext.Msg.alert(dSS._('Error'), dSS._('Could not save output mode settings'));
                this.fireEvent(eventName);
            }
        });
    },
    saveSettings: function (eventName) {
        var value = this.checkbox.getValue()
            ? dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL
            : dSS.config.deviceParams.OUTPUT_MODE_DISABLED;

        // do not submit anything if value was not loaded and also has
        // not been changed by the user
        if (!this.checkbox.isValid() || value === this.loadedValue) {
            this.fireEvent(eventName);
            return;
        }

        var dSUID = this.dSUID;

        if (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
            this.readTurnTimeCalibrationStatus(dSUID, function (result) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        throw 'error';
                    }
                    this.setOutputMode(
                        dSUID,
                        jsonData.result.value === 0
                            ? dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL_UNCALIBRATED
                            : dSS.config.deviceParams.OUTPUT_MODE_POSITION_CONTROL,
                        eventName
                    );
                } catch (err) {
                    Ext.Msg.alert(dSS._('Error'), dSS._('Could not save output mode settings'));
                    this.fireEvent(eventName);
                }
            }, eventName);
        } else {
            this.setOutputMode(dSUID, value, eventName);
        }
    },

    convertValueFromMagic: function (value) {
        return parseInt(value, 10);
    }
});

Ext.reg('dssdevgrkl300outputmodeform', dSS.devprops.GrKl300OutputModeForm);
