Ext.namespace("dSS", "dSS.config", "dSS.devprops", "dSS.gthelper");
// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.HeatingPropertiesForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.Msg.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        if (config.outputMode === undefined)  {
            Ext.Msg.alert(dSS._("Error"), ("Missing output mode"));
            return;
        }

        // hack #8607
        // some languages just can't keep it short and we can't use automatic
        // layouting because Ext does not support labels after the input fields
        // (i.e. %, s, etc.) which the PO wants to have
        var inputwidth = 150;
        var rightmarginhack = 55;
        if (dSS.config.language === 'fr_CH') {
            inputwidth = 180;
        } else if (dSS.config.language === 'pt_PT') {
            inputwidth = 185;
        } else if (dSS.config.language === 'da_DK') {
            inputwidth = 160;
            rightmarginhack = 65;
        } else if (dSS.config.language === 'it_IT') {
            inputwidth = 195;
        } else if (dSS.config.language === 'nl_NL') {
            inputwidth = 155;
        }

        var inputXPos =
            dSS.gthelper.getDialogConfig('heatingPropertyWindowWidth') -
            (inputwidth + rightmarginhack);

        this.dSUID = config.dSUID;
        this.outputMode = config.outputMode;

        var items = [];

        this.addEvents('formsubmitted', 'formloaded', 'forcereload',
                       'sharedevent', 'formretry', 'formchanged');

        items.push(new Ext.form.Label(
        {
            cls: 'dsClsHeatingPropertiesForm_ActuatorTypeLabel',

            text: dSS._("Actuator type") + ':',
            x: 6,
            y: 13
        }));

        this.actuator = new Ext.form.ComboBox(
        {
            cls: 'dsClsHeatingPropertiesForm_ActuatorCombo',

            typeAhead: false,
            lazyRender: true,
            mode: 'local',
            forceSelection: true,
            editable: false,
            triggerAction: 'all',
            store: new Ext.data.ArrayStore({
              id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [0, dSS._("Normally closed (NC)")],
                    [1, dSS._("Normally open (NO)")]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            width: inputwidth,
            x: inputXPos,
            y: 10
        });
        items.push(this.actuator);

        if (dSS.util.isHeatingPWMMode(config.outputMode)) {

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesForm_CyclicTimeOfPWMLabel',

                text: dSS._("Cyclic time of pulse width modulation (PWM)")+':',
                x: 6,
                y: 48
            }));

            this.cyclicTimePWM = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesForm_CyclicTimeOfPWMNumberField',

                bubbleEvents: [ 'change' ],
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 10,
                maxValue: 3600,
                width: inputwidth,
                x: inputXPos,
                y: 45
            });
            items.push(this.cyclicTimePWM);

            items.push(new Ext.form.Label(
            {
                text: dSS._("s"),
                x: inputXPos + inputwidth + 5,
                y: 48
            }));

            this.valveProtection = new Ext.form.Checkbox(
            {
                cls: 'dsClsHeatingPropertiesForm_DisableValveProtectionCheckbox',

                x: inputXPos - 2,
                y: 72
            });
            items.push(this.valveProtection);

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesForm_DisableValveProtectionLabel',

                text: dSS._("Disable valve protection") + ':',
                x: 6,
                y: 78
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesForm_CyclicTimeOfValveProtectionLabel',

                text: dSS._("Cyclic time of valve protection") + ':',
                x: 6,
                y: 107
            }));

            this.cyclicTimeValveProtection = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesForm_CyclicTimeOfValveProtectionNumberField',

                bubbleEvents: [ 'change' ],
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 10,
                maxValue: 3600,
                width: inputwidth,
                x: inputXPos,
                y: 104
            });
            items.push(this.cyclicTimeValveProtection);

            items.push(new Ext.form.Label(
            {
                text: dSS._("s"),
                x: inputXPos + inputwidth + 5,
                y: 107
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesForm_SettingValueInEmergencyModeLabel',

                text: dSS._("Setting value in emergency mode") + ':' ,
                x: 6,
                y: 142
            }));

            this.emergency = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesForm_SettingValueInEmergencyModeNumberField',

                bubbleEvents: [ 'change' ],
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 139
            });
            items.push(this.emergency);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 142
            }));
        }

        Ext.apply(this, {
            cls: 'dsClsHeatingPropertiesForm',

            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'forcereload', 'formretry',
                'formchanged'
            ],
            padding: 5,
            height: 175,
            scope: this,
            trackResetOnLoad: true,
            layout: 'absolute',
            items: items
        });

        this.loaded = {
            valve_control: false,
            valve_pwm: false,
            valve_timer: false
        };

        this.advanced =
        {
            ctrlClipMaxHigher:
            {
                loaded: undefined,
                modified: undefined,
            },
            ctrlClipMinLower:
            {
                loaded: undefined,
                modified: undefined,
            },
            ctrlClipMinZero:
            {
                loaded: undefined,
                modified: undefined,
            },
            pwmMaxX:
            {
                loaded: undefined,
                modified: undefined,
            },
            pwmMinX:
            {
                loaded: undefined,
                modified: undefined,
            },
            pwmMaxY:
            {
                loaded: undefined,
                modified: undefined,
            },
            pwmMinY:
            {
                loaded: undefined,
                modified: undefined,
            },
            pwmOffset:
            {
                loaded: undefined,
                modified: undefined,
            },
            switchedMax:
            {
                loaded: undefined,
                modified: undefined
            },
            switchedMin:
            {
                loaded: undefined,
                modified: undefined
            }
        }

        dSS.devprops.HeatingPropertiesForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
            this.actuator.on('select', function() {
                this.fireEvent('formchanged');
            }, this);

            if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                this.valveProtection.on('check', function() {
                    this.fireEvent('formchanged');
                },
                this);
            }

            this.on('change', function() {
                this.fireEvent('formchanged');
            },
            this);

            this.on('sharedevent', function(name, data) {
                if (name !== 'heatingpropadvancedchanged') {
                    return;
                }

                this.advanced[data.source].modified = data.value;
                this.fireEvent('formchanged');
            },
            this);
        },
        this);

        if (dSS.util.isHeatingPWMMode(this.outputMode)) {
            this.valveProtection.on('check', function(checkbox, checked) {
                if (checked === true) {
                    this.cyclicTimeValveProtection.setMinValue(0);
                    this.cyclicTimeValveProtection.setValue(0);
                    this.cyclicTimeValveProtection.disable();

                } else {
                    this.cyclicTimeValveProtection.enable();
                    this.cyclicTimeValveProtection.setMinValue(10);
                    this.cyclicTimeValveProtection.setValue(600);
                }
            },
            this);
        }

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                // only pwm control mode needs the valve timer
                if (!dSS.util.isHeatingPWMMode(this.outputMode)) {
                    this.loaded.valve_timer = true;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.VALVE_CONTROL_MODE:
                        this.loaded.valve_control = true;
                        if (data.json.ok === true) {
                            this.actuator.setValue(data.json.result.ctrlNONC);
                            this.actuator.originalValue =
                                data.json.result.ctrlNONC;

                            if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                                this.advanced.ctrlClipMaxHigher.loaded =
                                    data.json.result.ctrlClipMaxHigher;
                                this.advanced.ctrlClipMaxHigher.modified =
                                    data.json.result.ctrlClipMaxHigher;

                                this.advanced.ctrlClipMinLower.loaded =
                                    data.json.result.ctrlClipMinLower;
                                this.advanced.ctrlClipMinLower.modified =
                                    data.json.result.ctrlClipMinLower;

                                this.advanced.ctrlClipMinZero.loaded =
                                    data.json.result.ctrlClipMinZero;
                                this.advanced.ctrlClipMinZero.modified =
                                    data.json.result.ctrlClipMinZero;
                            }
                        } else {
                            this.actuator.markInvalid();
                        }

                        break;
                    case dSS.devprops.Loaders.VALVE_TIMER_MODE:
                        this.loaded.valve_timer = true;
                        if (!dSS.util.isHeatingPWMMode(this.outputMode)) {
                            return;
                        }
                        if (data.json.ok === true) {
                            var timer = data.json.result.valveProtectionTimer;
                            if (timer === 0) {
                                this.valveProtection.setValue(true);
                                this.cyclicTimeValveProtection.setMinValue(0);
                                this.cyclicTimeValveProtection.disable();
                            } else {
                                this.cyclicTimeValveProtection.setMinValue(10);
                                this.valveProtection.setValue(false);
                            }
                            this.cyclicTimeValveProtection.setValue(timer);
                            dSS.util.ResetDirtyField(this.cyclicTimeValveProtection);

                            this.emergency.setValue(data.json.result.emergencyValue);
                            dSS.util.ResetDirtyField(this.emergency);
                        } else {
                            this.valveProtection.markInvalid();
                            this.cyclicTimeValveProtection.markInvalid();
                            this.emergency.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.VALVE_PWM_MODE:
                        this.loaded.valve_pwm = true;
                        if (data.json.ok === true) {
                            if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                                this.advanced.pwmMaxY.loaded =
                                    data.json.result.pwmMaxY;
                                this.advanced.pwmMaxY.modified =
                                    data.json.result.pwmMaxY;

                                this.advanced.pwmMinY.loaded =
                                    data.json.result.pwmMinY;
                                this.advanced.pwmMinY.modified =
                                    data.json.result.pwmMinY;

                                this.cyclicTimePWM.setValue(data.json.result.pwmPeriod);
                                dSS.util.ResetDirtyField(this.cyclicTimePWM);
                            }

                            this.advanced.pwmMaxX.loaded =
                                data.json.result.pwmMaxX;
                            this.advanced.pwmMaxX.modified =
                                data.json.result.pwmMaxX;
                            this.advanced.pwmMinX.loaded =
                                data.json.result.pwmMinX;
                            this.advanced.pwmMinX.modified =
                                data.json.result.pwmMinX;
                            this.advanced.pwmOffset.loaded =
                                data.json.result.pwmOffset;
                            this.advanced.pwmOffset.modified =
                                data.json.result.pwmOffset;
                        } else {
                            if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                                this.cyclicTimePWM.markInvalid();
                            }
                        }
                        break;
                    default:
                        return;
                }

                if ((this.loaded.valve_pwm === true) &&
                    (this.loaded.valve_timer === true) &&
                    (this.loaded.valve_control === true)) {
                    this.fireEvent('formloaded', true);

                    this.loaded = {
                        valve_control: false,
                        valve_pwm: false,
                        valve_timer: false
                    };
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                if (this.getForm().isValid() !== true) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Please correct values in the red " +
                                        "marked fields and try again"));
                    this.fireEvent('formretry');
                    return;
                }

                var save = [];

                var params = this.getAdvancedParams(
                                    dSS.devprops.Loaders.VALVE_CONTROL_MODE);

                if (this.actuator.isDirty()) {
                    if (params === undefined) {
                        params = {};
                    }
                    params.ctrlNONC = this.actuator.getValue();
                }

                if (params !== undefined) {
                    params.dsuid = this.dSUID;

                    save.push({
                        url: '/json/device/setValveControlMode',
                        scope: this,
                        params: params,
                        failureMsg: dSS._("Could not save actuator type")
                    });
                }

                params = this.getAdvancedParams(
                                    dSS.devprops.Loaders.VALVE_PWM_MODE);

                if (dSS.util.isHeatingPWMMode(this.outputMode)) {

                    if (this.cyclicTimePWM.isDirty()) {
                        if (params === undefined) {
                            params = {};
                        }
                        params.pwmPeriod = this.cyclicTimePWM.getValue();
                    }

                    if (params !== undefined) {
                        params.dsuid = this.dSUID;
                        save.push({
                            url: '/json/device/setValvePwmMode',
                            scope: this,
                            params: params,
                            failureMsg: dSS._("Could not save valve PWM settings")
                        });
                    }

                    params = {};
                    if (this.cyclicTimeValveProtection.isDirty() ||
                        this.valveProtection.isDirty()) {
                        params.valveProtectionTimer =
                            this.cyclicTimeValveProtection.getValue();
                    }
                    if (this.emergency.isDirty()) {
                        params.emergencyValue = this.emergency.getValue();
                    }

                    if (this.cyclicTimeValveProtection.isDirty() ||
                        this.valveProtection.isDirty() ||
                        this.emergency.isDirty()) {
                        params.dsuid = this.dSUID;
                        save.push({
                            url: '/json/device/setValveTimerMode',
                            scope: this,
                            params: params,
                            failureMsg: dSS._("Could not save valve timer settings")
                        });
                    }
                }

                this.saveOK = true;

                var call = this.saveData.createDelegate(this, [ save, true ]);
                call();
            },
            this);
    },
    saveDataCallback: function(records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },
    saveData: function(records, ok) {
        if (ok === false) {
            this.saveOK = false;
        }
        if (records.length < 1) {
            if (this.saveOK === false) {
                dSS.MessageBus.fireEvent('statusmessage',
                    dSS._("Error") + ": " +
                    dSS._("Some properties could not be saved"));
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function(state) {
            var call = this.saveDataCallback.createDelegate(this,
                                                            [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    },
    getAdvancedParams: function(loader) {
        var params = {};
        var modified = false;
        if (loader === dSS.devprops.Loaders.VALVE_CONTROL_MODE) {
            if (this.advanced.ctrlClipMaxHigher.loaded !==
                this.advanced.ctrlClipMaxHigher.modified) {
                params.ctrlClipMaxHigher =
                    this.advanced.ctrlClipMaxHigher.modified;
                modified = true;
            }
            if (this.advanced.ctrlClipMinLower.loaded !==
                this.advanced.ctrlClipMinLower.modified) {
                params.ctrlClipMinLower =
                    this.advanced.ctrlClipMinLower.modified;
                modified = true;
            }
            if (this.advanced.ctrlClipMinZero.loaded !==
                this.advanced.ctrlClipMinZero.modified) {
                params.ctrlClipMinZero =
                    this.advanced.ctrlClipMinZero.modified;
                modified = true;
            }
        } else if (loader === dSS.devprops.Loaders.VALVE_PWM_MODE) {
            if (this.advanced.pwmMaxX.loaded !==
                this.advanced.pwmMaxX.modified) {
                params.pwmMaxX = this.advanced.pwmMaxX.modified;
                modified = true;
            }
            if (this.advanced.pwmMinX.loaded !==
                this.advanced.pwmMinX.modified) {
                params.pwmMinX = this.advanced.pwmMinX.modified;
                modified = true;
            }
            if (this.advanced.pwmMaxY.loaded !==
                this.advanced.pwmMaxY.modified) {
                params.pwmMaxY = this.advanced.pwmMaxY.modified;
                modified = true;
            }
            if (this.advanced.pwmMinY.loaded !==
                this.advanced.pwmMinY.modified) {
                params.pwmMinY = this.advanced.pwmMinY.modified;
                modified = true;
            }
            if (this.advanced.pwmOffset.loaded !==
                this.advanced.pwmOffset.modified) {
                params.pwmOffset = this.advanced.pwmOffset.modified;
                modified = true;
            }
        }

        if (modified !== true) {
            return undefined;
        }

        return params;
    }
});

Ext.reg('heatingpropsform', dSS.devprops.HeatingPropertiesForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.HeatingPropertiesAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;
        this.outputMode = config.outputMode;
        this.matrix = config.matrix;

        this.addEvents('formsubmitted', 'formloaded', 'sharedata', 'formretry',
                       'formchangd');

        var inputXPos = dSS.gthelper.getDialogConfig('heatingPropertyWindowWidth') - 205;

        var specialLabelWidth = dSS.gthelper.getDialogConfig('heatingPropertyWindowWidth');
        specialLabelWidth = specialLabelWidth -
                            (specialLabelWidth - inputXPos) - 30;

        var inputwidth = 150;

        var height = 330;

        var items = [];

        var img_x = dSS.gthelper.getDialogConfig('heatingPropertyWindowWidth') - 32;
        img_x = img_x / 2;
        img_x = img_x - 100;

        this.loaded = {
            valveControl: false,
            valvePWM: false
        };

        if (dSS.util.isHeatingPWMMode(this.outputMode)) {
            height = 510;
            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MaxPWMValueLabel',

                text: dSS._("Maximum PWM value") + ':',
                x: 6,
                y: 13
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg01red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 13

            }));

            this.maxPWM = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MaxPWMValueNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmMaxY',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 11
            });
            items.push(this.maxPWM);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 14
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MinPWMValueLabel',

                text: dSS._("Minimum PWM value") + ':',
                x: 6,
                y: 49
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg02red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 49
            }));

            this.minPWM = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MinPWMValueNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmMinY',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 46
            });
            items.push(this.minPWM);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 49
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MinSettingValueLabel',

                text: dSS._("Minimum setting value") + ':',
                x: 6,
                y: 84
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg03red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 84
            }));

            this.minSetVal = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MinSettingValueNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmMinX',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 81
            });
            items.push(this.minSetVal);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 84
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MaxSettingValueLabel',

                text: dSS._("Maximum setting value") + ':',
                x: 6,
                y: 119
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg04red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 119
            }));

            this.maxSetVal = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_MaxSettingValueNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmMaxX',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 116
            });
            items.push(this.maxSetVal);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 119
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OffsetLabel',

                text: dSS._("Offset") + ':',
                x: 6,
                y: 154
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg05grey',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 154
            }));

            this.offset = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OffsetNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmOffset',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: true,
                minValue: -100,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 151
            });
            items.push(this.offset);

            items.push(new Ext.form.Label(
            {
                xtype: 'label',
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 154
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueAboveMaxBox',

                html: dSS._("PWM value above max. setting value = 100%") + ':',
                x: 6,
                y: 189,
                height: 30,
                width: specialLabelWidth
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg06blue',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 189
            }));

            this.pwmAbove = new Ext.form.Checkbox(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueAboveMaxCheckbox',

                ref: 'ctrlClipMaxHigher',
                x: inputXPos,
                y: 184
            });
            items.push(this.pwmAbove);

            items.push(new Ext.BoxComponent(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueBelowMinBox',

                html: dSS._("PWM value below min. setting value = 0%") + ':',
                x: 6,
                y: 224,
                height: 30,
                width: specialLabelWidth
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg07blue',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 224
            }));


            this.pwmBelow = new Ext.form.Checkbox(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueBelowMinCheckbox',

                ref: 'ctrlClipMinLower',
                x: inputXPos,
                y: 219
            });
            items.push(this.pwmBelow);

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueEqualsZeroLabel',

                text: dSS._("PWM value = 0 for setting value = 0") + ':',
                x: 6,
                y: 259
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg08green',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 259
            }));

            this.pwmZero = new Ext.form.Checkbox(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_PWMValueEqualsZeroCheckbox',

                ref: 'ctrlClipMinZero',
                x: inputXPos,
                y: 254
            });
            items.push(this.pwmZero);

            items.push(new Ext.BoxComponent(
            {
                autoEl: { tag: 'img', src: '/images/graph_pwm.png' },
                x: img_x,
                y: 288
            }));
        } else {
            this.loaded.valveControl = true;

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_DisableOutputForSetValueLowerThanLabel',

                text: dSS._("Disable output for set value lower than") + ':',
                x: 6,
                y: 13
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg01red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 13

            }));

            this.outOff = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OutOffNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'switchedMin',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 11
            });
            items.push(this.outOff);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 14
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_EnableOutputForSetValueHigherThanLabel',
                
                text: dSS._("Enable output for set value higher than") + ':',
                x: 6,
                y: 49
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg02red',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 49
            }));

            this.outOn = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OutOnNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'switchedMax',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: false,
                minValue: 0,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 46
            });
            items.push(this.outOn);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 49
            }));

            items.push(new Ext.form.Label(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OffsetLabel',

                text: dSS._("Offset") + ':',
                x: 6,
                y: 84
            }));

            items.push(new Ext.BoxComponent(
            {
                cls: 'hg03grey',
                width: 16,
                height: 16,
                x: inputXPos - 20,
                y: 84
            }));

            this.offset = new Ext.form.NumberField(
            {
                cls: 'dsClsHeatingPropertiesAdvancedForm_OffsetNumberField',

                bubbleEvents: [ 'change' ],
                ref: 'pwmOffset',
                allowBlank: false,
                allowDecimals: false,
                allowNegative: true,
                minValue: -100,
                maxValue: 100,
                width: inputwidth,
                x: inputXPos,
                y: 81
            });
            items.push(this.offset);

            items.push(new Ext.form.Label(
            {
                text: dSS._("%"),
                x: inputXPos + inputwidth + 5,
                y: 84
            }));

            items.push(new Ext.BoxComponent(
            {
                autoEl: { tag: 'img', src: '/images/graph_hysteresis.png' },
                x: img_x,
                y: 120
            }));
        }

        Ext.apply(this, {
            cls: 'dsClsHeatingPropertiesAdvancedForm',

            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'sharedata', 'formretry',
                'formchanged'
            ],
            padding: 5,
            height: height,
            scope: this,
            border: false,
            layout: 'absolute',
            items: items

        });

        dSS.devprops.HeatingPropertiesAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                var result;
                if (data.json.ok === true) {
                    result = data.json.result;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.VALVE_CONTROL_MODE:
                        this.loaded.valveControl = true;
                        if ((this.outputMode ===
                            dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
                            (this.outputMode ===
                             dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL)) {
                            break;
                        }
                        if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                            if (data.json.ok === true) {
                                this.pwmAbove.setValue(result.ctrlClipMaxHigher);
                                dSS.util.ResetDirtyField(this.pwmAbove);

                                this.pwmBelow.setValue(result.ctrlClipMinLower);
                                dSS.util.ResetDirtyField(this.pwmBelow);

                                this.pwmZero.setValue(result.ctrlClipMinZero);
                                dSS.util.ResetDirtyField(this.pwmZero);
                            } else {
                                this.pwmAbove.markInvalid();
                                this.pwmBelow.markInvalid();
                                this.pwmZero.markInvalid();
                            }
                        }
                        break;
                    case dSS.devprops.Loaders.VALVE_PWM_MODE:
                        this.loaded.valvePWM = true;
                        if (dSS.util.isHeatingPWMMode(this.outputMode)) {
                            if (data.json.ok === true) {
                                this.maxPWM.setValue(result.pwmMaxY);
                                dSS.util.ResetDirtyField(this.maxPWM);

                                this.minPWM.setValue(result.pwmMinY);
                                dSS.util.ResetDirtyField(this.minPWM);

                                this.minSetVal.setValue(result.pwmMinX);
                                dSS.util.ResetDirtyField(this.minSetVal);

                                this.maxSetVal.setValue(result.pwmMaxX);
                                dSS.util.ResetDirtyField(this.maxSetVal);

                                this.offset.setValue(result.pwmOffset);
                                dSS.util.ResetDirtyField(this.offset);
                            } else {
                                this.maxPWM.markInvalid();
                                this.minPWM.markInvalid();
                                this.maxSetVal.markInvalid();
                                this.minSetVal.markInvalid();
                                this.offset.markInvalid();
                            }
                        } else { 
                            if (data.json.ok === true) {
                                this.outOff.setValue(result.switchedMin);
                                dSS.util.ResetDirtyField(this.outOff);

                                this.outOn.setValue(result.switchedMax);
                                dSS.util.ResetDirtyField(this.outOn);

                                this.offset.setValue(result.pwmOffset);
                                dSS.util.ResetDirtyField(this.offset);
                            } else {
                                this.outOff.markInvalid();
                                this.outOn.markInvalid();
                                this.offset.markInvalid();
                            }
                        }
                        break;
                    default:
                        return;
                }

                if ((this.loaded.valveControl === true) &&
                    (this.loaded.valvePWM === true)) {
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        if (dSS.util.isHeatingPWMMode(this.outputMode)) {
            this.maxPWM.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                                { source: me.ref, value: newval });
            },
            this);

            this.minPWM.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.pwmAbove.on('check', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.pwmBelow.on('check', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.pwmZero.on('check', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.maxSetVal.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.minSetVal.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);
        } else if ((this.outputMode ===
                        dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
                   (this.outputMode ===
                        dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL)) {

            this.outOff.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);

            this.outOn.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);
        }

        if (this.offset) {
            this.offset.on('change', function(me, newval) {
                this.fireEvent('sharedata', 'heatingpropadvancedchanged',
                               { source: me.ref, value: newval });
            },
            this);
        }


        this.on('submit', function() {
            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields in the advanced settings " +
                                    "tab and try again"));
                this.fireEvent('formretry');
                return;
            }

            var params = {};

            var modified = false;

            if (this.outOff && this.outOff.isDirty()) {
                params.switchedMin = this.outOff.getValue();
                modified = true;
            }

            if (this.outOn && this.outOn.isDirty()) {
                params.switchedMax = this.outOn.getValue();
                modified = true;
            }

            if (this.offset && this.offset.isDirty()) {
                params.pwmOffset = this.offset.getValue();
                modified = true;
            }

            if (modified) {
                params.dsuid = this.dSUID;
                dSS.util.genericRequest({
                    url: '/json/device/setValvePwmMode',
                    scope: this,
                    params: params,
                    failureMsg: dSS._("Could not save valve PWM settings"),
                    evt: 'formsubmitted'
                });
            } else {
                this.fireEvent('formsubmitted');
            }
        },
        this);
    }
});

Ext.reg('heatingpropsadvancedform', dSS.devprops.HeatingPropertiesAdvancedForm);

