/* globals dSS, Ext */

/* Button Function dropdown  - 1-way, 2-way up etc. */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured joker group)
dSS.devprops.MultiButtonConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function (config) {
        if (config.device == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        this.dSUID = config.device.get('dsuid');
        this.buttonConfig = config.device.get('buttonInputMode');
        this.buttonIndex = config.device.get('buttonInputIndex');
        this.buttonID = config.device.get('buttonID');
        this.group = config.group;
        this.matrix = config.matrix;

        this.deviceIsOfGrKl300Type = dSS.util.isGrKl300TypeDevice(config.device);

        var devInfo = dSS.util.parseProductID(config.device.get('productID'));
        this.deviceType = devInfo[0];

        this.forceSave = false;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var selectionItems = this.getSelectionItems(this.group);

        this.buttonInputTypeSelection = new Ext.form.ComboBox({
            cls: 'dsClsMultiButtonConfigForm_ButtonInputTypeSelectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            fieldLabel: dSS._('Button function'),
            labelStyle: 'margin-top: 5px',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: selectionItems
            }),
            valueField: 'id',
            displayField: 'displayText',
            disabled: this.buttonID === dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL ||
                this.group == dSS.config.groups.JOKER ||
                this.deviceIsOfGrKl300Type && this.deviceIsOfGrKl300Type.Hkl
        });

        Ext.apply(this, {
            cls: 'dsClsMultiButtonConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged', 'formsetdeviceconfig' ],
            padding: 5,
            height: 35,
            hidden: this.deviceType === dSS.config.deviceTypeNum.UMR && this.group === dSS.config.groups.JOKER,
            scope: this,
            items: [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items: [
                        this.buttonInputTypeSelection
                    ]
                }
            ]
        });

        dSS.devprops.MultiButtonConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.buttonInputTypeSelection.on('select', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        dSS.MessageBus.addListener('pushbuttonhighlevelchanged', this.highlevelChanged, this);

        dSS.MessageBus.addListener('jokerconfigformchanged', this.updateList, this);

        this.on('afterrender', function () {
            this.fireEvent('formloaded', true);
        }, this);


        this.buttonInputTypeSelection.getStore().on('load', function () {
            this.setValue(this.getValue()); // update screen
        }, this.buttonInputTypeSelection);

        this.translateButtonconfigToSelection = function (buttonConfig) {
            if (buttonConfig >= dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1 &&
                buttonConfig <= dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN;
            } else if (buttonConfig >= dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1 &&
                buttonConfig <= dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP;
            } else if (buttonConfig === dSS.config.deviceParams.BUTTONINPUT_2WAY) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY;
            } else if (buttonConfig === dSS.config.deviceParams.BUTTONINPUT_1WAY) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_1WAY_COMBINED;
            } else {
                return dSS.config.ButtonInputMode.BUTTONINPUT_1WAY;
            }
        };

        this.buttonInputTypeSelection.on('afterrender', function () {
            if (!this.closed) {
                this.buttonInputTypeSelection.setValue(this.translateButtonconfigToSelection(this.buttonConfig));
            }
        }, this);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('beforedestroy', function () {
            this.closed = true;
            dSS.MessageBus.removeListener('pushbuttonhighlevelchanged', this.highlevelChanged, this);
            dSS.MessageBus.removeListener('jokerconfigformchanged', this.updateList, this);
            dSS.MessageBus.removeListener('inputtypechanged', this.updateVisibility, this);
        }, this);

        this.on('submit', function () {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            var value = this.buttonInputTypeSelection.getValue();
            var dSUID = this.dSUID;
            var buttonConfig = this.buttonConfig;
            if (!this.forceSave && (value === '' || value == this.translateButtonconfigToSelection(buttonConfig))) {
                this.fireEvent('formsubmitted');
                return;
            }

            if (this.matrix.setumr200config) {
                this.fireEvent('formsetdeviceconfig', {
                        pushButtonMode: dSS.config.ButtonInputModeV2Map[value]
                });
                return;
            }

            Ext.Ajax.request({
                url: '/json/device/setButtonInputMode',
                method: 'GET',
                disableCaching: true,
                scope: this,
                params: {
                    dsuid: dSUID,
                    mode: value
                },
                success: function (result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (!jsonData.ok) {
                            throw 'error';
                        }

                        var reply = jsonData.result;
                        var store = dSS.data.Loader.getDeviceStore();

                        var index = store.find('dsuid', reply.update.dSUID);
                        if (index > -1) {
                            var d = store.getAt(index);
                            d.set('buttonInputMode', reply.update.buttonInputMode);
                            d.commit();
                        }

                        dSS.util.processDeviceActionResponse(reply);
                        this.fireEvent('formsubmitted');
                    } catch (err) {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not set button input mode of device')
                        );
                        this.fireEvent('formsubmitted');
                    }
                },
                failure: function (result, request) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not set button input mode of device')
                    );
                    this.fireEvent('formsubmitted');
                }
            });
        }, this);
    },

    highlevelChanged: function (highlevelActive) {
        if (highlevelActive) {
            this.buttonInputTypeSelection.setValue(dSS.config.ButtonInputMode.BUTTONINPUT_1WAY);
        }
        this.buttonInputTypeSelection.setDisabled(highlevelActive);
    },

    updateVisibility: function (input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.hide();
            this.disable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.forceSave = true;
            this.enable();
            this.updateList(this.group);
        }
        this.fireEvent('updateshadow');
    },

    updateList: function (group) {
        this.doLayout();
        if (this.group === dSS.config.groups.JOKER && group !== dSS.config.groups.JOKER) {
            this.buttonInputTypeSelection.setValue(dSS.config.ButtonInputMode.BUTTONINPUT_1WAY);
        }
        this.group = group;
        var store = this.buttonInputTypeSelection.getStore();
        store.loadData(this.getSelectionItems(group), false);

        if (this.deviceType === dSS.config.deviceTypeNum.UMR || this.deviceType === dSS.config.deviceTypeNum.SKM) {
            if (group === dSS.config.groups.JOKER) {
                if (this.isVisible()) {
                    this.hide();
                }
            } else if (!this.isVisible()) {
                this.show();
            }
        }
    },

    getSelectionItems: function (group) {
        switch (true) {
            case this.matrix.pushbcombined:
                return [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY_COMBINED,
                        dSS._('1-way')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY,
                        dSS._('2-way')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                        dSS._('1-way + joker')
                    ]
                ];
            case Boolean(this.deviceIsOfGrKl300Type):
                return [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                        dSS._('1-way + sensor')
                    ], [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP,
                        dSS._('2-way (up/in)')
                    ], [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN,
                        dSS._('2-way (down/out)')
                    ]
                ];
            case group === dSS.config.groups.SHADOW:
                return [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                        dSS._('1-way')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN,
                        dSS._('2-way down')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP,
                        dSS._('2-way up')
                    ]
                ];
            default:
                return [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                        dSS._('1-way')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN,
                        dSS._('2-way off')
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP,
                        dSS._('2-way on')
                    ]
                ];
        }
    }
});

Ext.reg('dssdevmultibuttonconfigform', dSS.devprops.MultiButtonConfigForm);
