Ext.namespace("dSS", "dSS.config", "dSS.devprops");

dSS.devprops.CheckShowSceneProperties = function(matrix) {
    return (matrix.blink || matrix.dontcare);
};

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.ScenePropertiesForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.deviceRecord;
        var pid = dSS.util.parseProductID(this.device.get('productID'));
        var devType = dSS.config.deviceTypes[pid[0]];

        this.dSUID = this.device.get('dsuid');
        this.scene = config.scene;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var checkboxes = [];

        if (config.visibilityMatrix === undefined) {
            Ext.Msg.alert(("Error"), ("Missing config options"));
            return;
        }

        var blinkstring = "Blink when calling scene \"%1\"";
        if (devType === 'UMR') {
            blinkstring = "Impulse when calling scene \"%1\"";
        }

        checkboxes.push({
            cls: 'dsClsScenePropertiesForm_StandardSettingsCheckboxGroup_BlinkCheckbox',

            boxLabel: dSS.gt.strargs(dSS._(blinkstring),
                          [ dSS._(dSS.config.sceneNames[this.scene]) ]),
            disabled: !config.visibilityMatrix.blink,
            name: 'cb_sceneprops_' + config.dSUID,
            id: 'blink'
        });

        checkboxes.push({
            cls: 'dsClsScenePropertiesForm_StandardSettingsCheckboxGroup_DontCareCheckbox',

            boxLabel: dSS.gt.strargs(dSS._("Retain output when calling " +
                                "scene \"%1\""),
                                [ dSS._(dSS.config.sceneNames[this.scene]) ]),
            name: 'cb_sceneprops_' + config.dSUID,
            disabled: !config.visibilityMatrix.dontcare,
            id: 'dontcare',
            listeners:
            {
                afterrender: function(box) {
                    if (this.disabled) {
                        return;
                    }
                    this.dontCareOwnerID = box.ownerCt.id;
                    Ext.QuickTips.register({
                        target: this.dontCareOwnerID,
                        text: dSS._("Do not change output of the device " +
                                    "when scene is called, i.e. do not react " +
                                    "to the scene call"),
                        width: 400,
                        trackMouse: false,
                        dismissDelay: 10000
                    });
                },
                beforedestroy: function(box) {
                    if (this.disabled) {
                        return;
                    }

                    if (this.dontCareOwnerID !== undefined) {
                        Ext.QuickTips.unregister(this.dontCareOwnerID);
                    }
                },
                check: function(box, checked) {
                   dSS.MessageBus.fireEvent('deviceoutputmodechanged', checked);
                }
            }
        });

        this.standardSettings = new Ext.form.CheckboxGroup({
            cls: 'dsClsScenePropertiesForm_StandardSettingsCheckboxGroup',

            style: { marginLeft: '10px' },
            columns: 1,
            items: checkboxes
        });
 
        this.canSubmit = false;

        this.outModeTip = new Ext.ToolTip({
            cls: 'dsClsScenePropertiesForm_ConfigNotPossibleToolTip',

            html: dSS._("Configuration not possible: output" +
                  " mode of the device is disabled"),
            draggable: true,
            title: dSS._("Notice"),
            anchor: 'left',
            dismissDelay: 10000
        });

        Ext.apply(this, {
            cls: 'dsClsScenePropertiesForm',

            layout: 'fit',
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            scope: this,
            bodyCfg: {
                cls: 'x-window-mc'
            },
            items: this.standardSettings
        });

        dSS.devprops.ScenePropertiesForm.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('loadcomplete', function() {
                if (this.disabled) {
                    return;
                }
                this.standardSettings.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            if (this.canSubmit !== true) {
                this.fireEvent('formsubmitted');
                return;
            }
            var value = {};

            if (this.standardSettings.items.get('blink').getValue() === true) {
                value.flashMode = 1;
            } else {
                value.flashMode = 0;
            }

            if (this.standardSettings.disabled === false) {
                if (this.standardSettings.items.get('dontcare') !== undefined) {
                    if (this.standardSettings.items.get('dontcare').getValue()
                                                                     === true) {
                        value.dontCare = 1;
                    } else {
                        value.dontCare = 0;
                    }
                }
            }

            // setting might be disabled for some devices
            if (value.dontCare === undefined) {
                value.dontCare = this.loadedValueObj.dontCare;
            }
            if (value.flashMode === undefined) {
                value.flashMode = this.loadedValueObj.flashMode;
            }

            if ((value.dontCare == this.loadedValueObj.dontCare) &&
                (value.flashMode == this.loadedValueObj.flashMode)) {
                this.fireEvent('formsubmitted');
                return;
            }

            // update local cache
            this.loadedValueObj.dontCare = value.dontCare;
            this.loadedValueObj.flashMode = value.flashMode;

            value.dsuid = this.dSUID;
            value.sceneID = this.scene;

            dSS.util.genericRequest({
                url: '/json/device/setSceneMode',
                params: value,
                failureMsg: dSS._("Could not save device scene settings"),
                scope: this,
                evt: 'formsubmitted'
            });
        },
        this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.SCENE_MODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    Ext.Msg.alert(dSS._("Error"), data.json.message);
                    this.fireEvent('formloaded', this.canSubmit);
                    return;
                }

                var val = data.json.result;

                this.loadedValueObj = val;
                this.standardSettings.setValue([val.flashMode,
                                                 val.dontCare]);

                this.canSubmit = true;
                if (this.device.get('outputMode') ==
                        dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
                        this.outModeTip.initTarget(this.standardSettings.getId());
                        this.outModeTip.enable();
                        this.outModeTip.show();
                        this.standardSettings.disable();
                }
                this.fireEvent('formloaded', this.canSubmit);
            },
            this);
    }
});

Ext.reg('dssdevscenepropsform', dSS.devprops.ScenePropertiesForm);

dSS.devprops.ScenePropertiesAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.deviceRecord;

        this.dSUID = this.device.get('dsuid');
        this.scene = config.scene;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var checkboxes = [];

        if (config.visibilityMatrix === undefined) {
            Ext.Msg.alert(("Error"), ("Missing config options"));
            return;
        }

        // advanced properties available
        if ((config.visibilityMatrix.ledauto === true) ||
            (config.visibilityMatrix.leddark === true) ||
            (config.visibilityMatrix.transt === true)) {

            var acbitems = [];

            var leditems = [];

            if (config.visibilityMatrix.ledauto === true) {
                leditems.push(
                    {
                        cls: 'dsClsScenePropertiesForm_LedSettingsRadioGroup_AutoRadio',

                        boxLabel: dSS._("auto"),
                        name: 'rb_led_sceneprops_' + config.dSUID,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_AUTO_DARK,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_AUTO_DARK
                    });
            }

            if (config.visibilityMatrix.leddark === true) {
                leditems.push(
                    {
                        cls: 'dsClsScenePropertiesForm_LedSettingsRadioGroup_OnRadio',

                        boxLabel: dSS._("on"),
                        name: 'rb_led_sceneprops_' + config.dSUID,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_ON,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_ON
                    });
                leditems.push(
                    {
                        cls: 'dsClsScenePropertiesForm_LedSettingsRadioGroup_DarkRadio',

                        boxLabel: dSS._("dark"),
                        name: 'rb_led_sceneprops_' + config.dSUID,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_AUTO_DARK,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_AUTO_DARK
                    });
            }

            if ((config.visibilityMatrix.ledauto === true) ||
                (config.visibilityMatrix.leddark === true)) {

                leditems.push(
                    {
                        cls: 'dsClsScenePropertiesForm_LedSettingsRadioGroup_OffRadio',

                        boxLabel: dSS._("off"),
                        name: 'rb_led_sceneprops_' + config.dSUID,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_OFF,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_OFF
                    });

                this.ledSettings = new Ext.form.RadioGroup({
                    cls: 'dsClsScenePropertiesForm_LedSettingsRadioGroup',

                    columns: 1,
                    padding: 5,
                    fieldLabel: dSS._("LED mode"),
                    items: leditems
                });

                acbitems.push(this.ledSettings);
            }

            if (config.visibilityMatrix.transt === true) {
                var transitionsettings =
                [
                    {
                        cls: 'dsClsScenePropertiesForm_DimSettingsRadioGroup_StandardRadio',

                        boxLabel: dSS._("standard"),
                        name: 'rb_dim_sceneprops_' + config.dSUID,
                        id: 'dim' +
                         dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD,
                        inputValue:
                         dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD
                    },
                    {
                        cls: 'dsClsScenePropertiesForm_DimSettingsRadioGroup_SlowRadio',

                        boxLabel: dSS._("slow"),
                        name: 'rb_dim_sceneprops_' + config.dSUID,
                        id: 'dim' +
                            dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW,
                        inputValue:
                            dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW
                    }
                ];

                if (config.visibilityMatrix.customtransitiontime === true) {
                    transitionsettings.push(
                    {
                        cls: 'dsClsScenePropertiesForm_DimSettingsRadioGroup_CustomRadio',

                        boxLabel: dSS._("custom"),
                        name: 'rb_dim_sceneprops_' + config.dSUID,
                        id: 'dim' +
                           dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM,
                        inputValue:
                           dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM
                    });
                }

                this.dimSettings = new Ext.form.RadioGroup({
                    cls: 'dsClsScenePropertiesForm_DimSettingsRadioGroup',

                    columns: 1,
                    padding: 5,
                    fieldLabel: dSS._("Transition time"),
                    items: transitionsettings
                });

                acbitems.push(this.dimSettings);
            }
        }

        this.canSubmit = false;

        Ext.apply(this, {
            cls: 'dsClsScenePropertiesAdvancedForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            bodyCfg: {
                cls: 'x-window-mc'
             },
            items: acbitems
        });

        dSS.devprops.ScenePropertiesAdvancedForm.superclass.constructor.apply(
                this, arguments);

        this.on('loadcomplete', function() {
                // HACK!!!
                // somehow ext eventing seems to get out of order under certain
                // conditions; when it happens setValue() on a radio group does
                // trigger a change event; even though the event is triggered
                // *before* we subscribe in loadcomplete, we are still called
                // with it, which results in a 'change' event that was not
                // user initiated but actually a result of the 'dataloaded'
                // event earlier on
                //
                // I could not find out the cause for it, but as a workaround
                // it seems to be enough to delay the subscription to the
                // change events by half a second

                var subscribeLater = function() {
                    if (this.ledSettings !== undefined) {
                        this.ledSettings.on('change', function() {
                            this.fireEvent('formchanged');
                        },
                        this);
                    }

                    if (this.dimSettings !== undefined) {
                        this.dimSettings.on('change', function() {
                            this.fireEvent('formchanged');
                        },
                        this);
                    }
                };
                subscribeLater.defer(500, this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.SCENE_MODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    Ext.Msg.alert(dSS._("Error"), data.json.message);
                    this.fireEvent('formloaded', this.canSubmit);
                    return;
                }

                var val = data.json.result;

                this.loadedValueObj = val;
                if (this.ledSettings !== undefined) {
                    this.ledSettings.setValue('led' + val.ledconIndex, true);
                }

                if (this.dimSettings !== undefined) {
                    this.dimSettings.setValue('dim' + val.dimtimeIndex, true);
                }

                this.canSubmit = true;
                this.fireEvent('formloaded', this.canSubmit);
            },
            this);

        this.on('submit', function() {
            if (this.canSubmit !== true) {
                this.fireEvent('formsubmitted');
                return;
            }
            var value = {};

            // If this.ledSettings was never made visible (user didn't go to Advanced Settings tab),
            // this.ledSettings.getValue() will return null even though this.ledSettings.setValue() was
            // correctly executed when data was fetched (value was set on the radio button group).
            // In such case we can assume the setting didn't change.
            if (this.ledSettings !== undefined && this.ledSettings.getValue() != null) {
                var ledval = this.ledSettings.getValue().getRawValue();
                if ((ledval ===
                    dSS.config.deviceParams.LED_CONFIG_AUTO_DARK.toString()) ||
                    (ledval ===
                        dSS.config.deviceParams.LED_CONFIG_OFF.toString()) ||
                    (ledval ===
                        dSS.config.deviceParams.LED_CONFIG_ON.toString())) {
                    value.ledconIndex = ledval;
                } else {
                    value.ledconIndex = this.loadedValueObj.ledconIndex;
                }
            } else {
                value.ledconIndex = this.loadedValueObj.ledconIndex;
            }

            // Same situation here.
            if (this.dimSettings !== undefined && this.dimSettings.getValue() != null) {
                var dimval = this.dimSettings.getValue().getRawValue();

                if ((dimval ===
                    dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD.toString()) ||
                    (dimval ===
                        dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW.toString()) ||
                    (dimval ===
                        dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM.toString())) {
                    value.dimtimeIndex = dimval;
                } else {
                    value.dimtimeIndex = this.loadedValueObj.dimtimeIndex;
                }
            } else {
                value.dimtimeIndex = this.loadedValueObj.dimtimeIndex;
            }

            if ((value.ledconIndex == this.loadedValueObj.ledconIndex) &&
                (value.dimtimeIndex == this.loadedValueObj.dimtimeIndex)) {
                this.fireEvent('formsubmitted');
                return;
            }

            value.dsuid = this.dSUID;
            value.sceneID = this.scene;

            dSS.util.genericRequest.defer(1000, this,
            [
                {
                    url: '/json/device/setSceneMode',
                    params: value,
                    failureMsg: dSS._("Could not save device scene settings"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('dssdevscenepropsadvancedform', dSS.devprops.ScenePropertiesAdvancedForm);
