/* globals Ext, dSS */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');
dSS.devprops.VisibilityMatrix = {
    GE: {},
    GN: {},
    RT: {},
    GR: {},
    BL: {},
    TK: {},
    SW: {}
};

dSS.devprops.ErrorMatrix = {};

dSS.devprops.GetVisibleProperties = function (record) {
    var none = dSS.devprops.ErrorMatrix;

    if (!record) {
        throw none;
    }

    var device = dSS.util.getDeviceInfoFromRecord(record);
    var modelUID = record.get('VdcModelUID');
    var dc = dSS.devprops.VisibilityMatrix[device.color];
    if (dc == null) {
        throw none;
    }

    var ret;
    if (String(modelUID).length) {
        ret = dc[modelUID];
        if (ret && ret[0]) {
            return ret[0];
        } else {
            throw none;
        }
    }

    var key = device.type + ':' + device.number;
    ret = dc[key];
    if (ret == null) {
        // try wildcard - device.typeId NNx
        key = key.slice(0, -1);
        ret = dc[key];
        if (ret == null) {
            // try wildcard - device.typeId Nxx
            key = key.slice(0, -1);
            ret = dc[key];
        }
    }

    if (ret == null || !ret.length) {
        throw none;
    }

    // select the set of flags for given dSUIDIndex
    var dsuidIndex = record.get('dSUIDIndex');
    ret = ret[dsuidIndex] || ret[0];

    var copied = false;
    var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret)); // deep clone
    // apply overrides
    if (device.typeId === dSS.config.deviceTypeNum.UMR) {
        // only dSUID 1. and 2. have input mode settings
        if (dsuidIndex > 1) {
            copy.optypeconfig = false;
            copy.pushbutton = false;
            copy.pushbadvanced = false;
            copy.twowayconfig = false;
            copy.akmdelay = false;
            copy.akmsensor = false;
            copy.akminput = false;

            // hack to allow to configure the temperature control group
            // on the new UMR in the joker configuration dialog, for
            // firmware versions 3.8.3 and later

            if (record.get('productRevision') >= 0x383) {
                copy.jokertempcontrol = true;

                if (dsuidIndex === 2) {
                    copy.umroutmodeswitch2x = true;
                }

                if (dSS.util.getJokerGroup(record.get('groups')) === dSS.config.groups.TEMPCONTROL) {
                    copy.heatingprops = true;
                }
                copy.valvetype = true;
            }
        }
        if (dsuidIndex < 2) {
            copy.blinkconfig = false;
            copy.umroutmode = false;
            copy.outvalue8 = false;
            copy.impulseconfig = false;
            copy.outconfigswitch = false;
            copy.enabletypehack = true;
        }
        ret = copy;
        copied = true;
    } else if (device.typeId === dSS.config.deviceTypeNum.UMV) {
        // only dSUID 1. has umvrelay and extradimmer settings
        if (dsuidIndex > 0) {
            copy.umvrelay = false;
        }
        ret = copy;
        copied = true;
    } else if (
        device.typeId === dSS.config.deviceTypeNum.SK &&
        device.classId === dSS.config.deviceClass.BL &&
        device.number === 204       // eslint-disable-line no-magic-numbers
    ) {
        // only the first dSUID has the full range of FTW settings
        if (dsuidIndex > 0) {
            copy.ftwtempcontrolventilationselect = false;
            copy.ftwdisplaysettings = false;
            copy.ftwbacklighttimeout = false;
            copy.temperatureoffset = false;
            ret = copy;
            copied = true;
        }
    } else if (device.typeId === dSS.config.deviceTypeNum.AKM) {
        copy.enabletypehack = true;
        copy.inputtypeonlysensor = true;
        ret = copy;
        copied = true;
    } else if (
        device.typeId === dSS.config.deviceTypeNum.TKM &&
        device.number === 300           // eslint-disable-line no-magic-numbers
    ) {
        copy.enabletypehack = true;
        copy.inputtypeonlysensor = false;
        ret = copy;
        copied = true;
    } else if (
        device.typeId === dSS.config.deviceTypeNum.SKM &&
        device.number === 300           // eslint-disable-line no-magic-numbers
    ) {
        copy.enabletypehack = true;
        copy.inputtypeonlysensor = false;
        ret = copy;
        copied = true;
    }

    var overrideMF = record.get('modelFeatures');
    if (overrideMF != null) {
        var overriden = ret;
        if (!copied) {
            overriden = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
        }
        Ext.apply(overriden, overrideMF);
        ret = overriden;
    }
    return ret;
};
