//= require <dSS/devices/device-settings/device-type-panels/DeviceSettingsHSVView>
//= require <dSS/devices/device-settings/device-type-panels/DeviceSettingsHSVViewApartmentMode>

/* globals Ext, dSS */

Ext.namespace('dSS', 'dSS.devices', 'dSS.config', 'dSS.gthelper');

// devClass
// stop button will be shown depending on the device class
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function (config) {
        this.addEvents('loading');
        this.notFirstCall = false;
        this.dSUID = config.dSUID;
        this.scene = config.scene;
        this.dummy = false;
        if (config.dummy != null) {
            this.dummy = config.dummy;
        }

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.apartmentMode = false;
        if (config.apartmentMode != null) {
            this.apartmentMode = config.apartmentMode;
        }

        this.matrix = config.matrix;
        this.virtualDevice = config.virtualDevice;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 210;
        this.EXTRA_HEIGHT_GRKL387HACK = this.EXTRA_HEIGHT + 25;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS = 332;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS_TEMPERATURE_ONLY = 240;
        this.EXTRA_HEIGHT_FCU = 246;
        var initialHeight = this.DEFAULT_HEIGHT;
        if (this.matrix.fcu) {
            initialHeight = this.EXTRA_HEIGHT_FCU;
        } else if (this.matrix.shadebladeang) {
            initialHeight = this.EXTRA_HEIGHT;
        } else if (this.matrix.outputchannels) {
            initialHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
        } else if (this.apartmentMode && this.matrix.grkl387workaround) {
            initialHeight = this.EXTRA_HEIGHT_GRKL387HACK;
        }

        if (config.overrideHeight != null) {
            this.EXTRA_HEIGHT = config.overrideHeight;
            initialHeight = config.overrideHeight;
        }

        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        var xPosOffset = 280;

        this.noconfig = new Ext.Panel({
            cls: 'dsClsDeviceSettingsPanel_NoConfig',

            x: 0,
            y: 0,
            border: false
        });

        var colorPickerConfig = {
            x: 0,
            y: 0,
            dSUID: this.dSUID
        };
        this.colorPicker = this.apartmentMode
            ? new dSS.devices.DeviceSettingsHSVViewApartmentMode(colorPickerConfig)
            : new dSS.devices.DeviceSettingsHSVView(colorPickerConfig);

        this.videoView = new dSS.devices.DeviceSettingsVideoView({
            x: 0,
            y: 0,
            initialHeight: initialHeight,
            dSUID: this.dSUID,
            apartmentMode: this.apartmentMode,
            device: config.device
        });

        this.configLink = new dSS.devices.DeviceSettingsVdcConfigLinkView({
            x: 0,
            y: 0,
            initialHeight: this.DEFAULT_HEIGHT,
            dSUID: this.dSUID,
            device: config.device
        });

        this.fcu = new dSS.devices.DeviceSettingsFCUView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: this.EXTRA_HEIGHT_FCU,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
            device: config.device
        });

        this.sliderView = new dSS.devices.DeviceSettingsDefaultView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: initialHeight,
            matrix: this.matrix,
            maxValue: this.maxValue,
            decPrecision: decPrecision,
            devClass: config.devClass,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
            device: config.device
        });

        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsPanel',

            layout: 'card',
            activeItem: 0,
            padding: 0,
            bubbleEvents: [ 'updateshadow', 'valuechanged', 'enableme', 'disableme', 'formchanged' ],
            height: initialHeight,
            items: [
                // do not reorder!
                this.noconfig,
                this.sliderView,
                this.colorPicker,
                this.fcu,
                this.configLink,
                this.videoView
            ]
        });

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.addEvents('updateshadow');

        this.on('afterrender', function () {
            this.fireEvent('updateshadow', initialHeight);
            this.setData(this.dSUID, config.devClass, config.present, config.matrix, config.virtualDevice, config.device);
        }, this);

        this.on('beforedestroy', function () {
            this.killRequest();
            this.closed = true;
        }, this);

        this.on('enableme', function () {
            if (!this.apartmentMode) {
                this.enable();
            }
        }, this);
        this.on('disableme', function () {
            if (!this.apartmentMode) {
                this.disable();
            }
        }, this);
    },

    visualConfig: function (matrix, record) {
        if (!this.apartmentMode) {
            this.fireEvent('disableme');
        }

        var device = dSS.util.getDeviceInfoFromRecord(record);
        if (matrix.customactivityconfig) {
            this.getLayout().setActiveItem(0);
        } else if (device.classId === dSS.config.deviceClass.MG && device.isVdcDevice) {
            this.getLayout().setActiveItem(5); // video view
        } else if ((matrix.outvalue8 || matrix.shadeposition) &&
            (!matrix.outputchannels) && (!matrix.fcu)) {
            this.getLayout().setActiveItem(1); // default slider view
        } else if (matrix.outputchannels) {
            this.getLayout().setActiveItem(2); // color picker
        } else if (matrix.fcu) {
            this.getLayout().setActiveItem(3); // fcu view
        } else if (device.isVdcDevice) {
            this.getLayout().setActiveItem(4); // vdc configuration link
        } else {
            this.getLayout().setActiveItem(0);
        }

        this.doLayout(false, true);
    },

    setData: function (dSUID, devClass, present, matrix, virtualDevice, record) {
        this.dSUID = dSUID;
        this.matrix = matrix;

        if (this.apartmentMode && matrix.grkl387workaround) {
            if (this.getHeight() !== this.EXTRA_HEIGHT_GRKL387HACK) {
                this.setHeight(this.EXTRA_HEIGHT_GRKL387HACK);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_GRKL387HACK);
            }
        } else if (matrix.shadebladeang) {
            if (this.getHeight() !== this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        } else if (matrix.outputchannels) {
            var supportsHSVChannels = record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'hue' || channel.channelId === 'saturation';
            });
            var supportsTemperatureChannel = record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'colortemp';
            });
            var newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
            if (!supportsHSVChannels) {
                newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS_TEMPERATURE_ONLY - 15;    // there'll be no "Switch to other view" label
            } else if (!supportsTemperatureChannel) {
                newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS - 15;     // there'll be no "Switch to other view" label
            }
            if (this.getHeight() !== newHeight) {
                this.setHeight(newHeight);
                this.fireEvent('updateshadow', newHeight);
            }
        } else if (matrix.fcu) {
            if (this.getHeight() !== this.EXTRA_HEIGHT_FCU) {
                this.setHeight(this.EXTRA_HEIGHT_FCU);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_FCU);
            }
        // } else if (virtualDevice === true) {
        //     if (this.getHeight() !== this.EXTRA_HEIGHT + 15) {
        //         this.setHeight(this.EXTRA_HEIGHT);
        //         this.fireEvent('updateshadow', this.EXTRA_HEIGHT + 15);
        //     }
        } else if (this.getHeight() !== this.DEFAULT_HEIGHT) {
            this.setHeight(this.DEFAULT_HEIGHT);
            this.fireEvent('updateshadow', this.DEFAULT_HEIGHT);
        }

        this.clearData();

        var bl361Hack = false;
        var device = dSS.util.getDeviceInfoFromRecord(record);
        if (devClass === dSS.config.deviceClass.BL) {
            if (
                device.type === 'KM' && device.number === 200 &&
                matrix.pwmvalue &&
                record.get('productRevision') == 0x361 &&
                record.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED
            ) {
                bl361Hack = true;
            }
        }

        this.sliderView.visualConfig(matrix, record, bl361Hack);
        this.visualConfig(matrix, record);

        if (!present) {
            return;
        }

        if (devClass === dSS.config.deviceClass.MG && virtualDevice) {
            this.videoView.loadData(dSUID, record, !this.notFirstCall);
        } else if (
            (matrix.outvalue8 || matrix.shadeposition) &&
            !matrix.outputchannels &&
            !matrix.fcu
        ) {
            this.sliderView.loadData(dSUID, devClass, matrix, virtualDevice, bl361Hack, record);
        } else if (matrix.outputchannels) {
            this.colorPicker.loadData(dSUID, record);
        } else if (matrix.fcu) {
            this.fcu.loadData(dSUID);
        } else if (virtualDevice) {
            this.configLink.loadData(dSUID, record, !this.notFirstCall);
        }

        this.doLayout(false, true);
        this.notFirstCall = true;
    },

    killRequest: function () {
        this.sliderView.killRequests();
        this.colorPicker.killRequests();
        this.videoView.killRequests();
        this.configLink.killRequests();
        this.fcu.killRequests();
    },

    clearData: function () {
        this.sliderView.clearData();
        this.colorPicker.clearData();
        this.fcu.clearData();
    },

    doPreview: function () {
        this.colorPicker.doPreview();
        this.sliderView.doPreview();
        this.fcu.doPreview();
        this.videoView.doPreview();
    },

    saveSceneValues: function () {
        if (!this.apartmentMode) {
            return;
        }
        this.colorPicker.saveSceneValues(this.scene);
        this.sliderView.saveSceneValues(this.scene);
        this.fcu.saveSceneValues(this.scene);
        this.videoView.saveSceneValues(this.scene);
    },

    resetDefaults: function () {
        this.sliderView.resetDefaults();
        this.colorPicker.resetDefaults();
        this.fcu.resetDefaults();
        this.videoView.resetDefaults();
    },

    resetValues: function () {
        this.sliderView.resetValues();
        this.colorPicker.resetValues();
        this.fcu.resetValues();
        this.videoView.resetValues();
    }

});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);
