//= require <dSS/devices/shade-property-dialogs/ShadeCalibrationWizard>
//= require <dSS/devices/shade-property-dialogs/GRKL300ShadeCalibrationWizard>
//= require <dSS/devices/shade-property-dialogs/FinsCalibrationWizard>
//= require <dSS/devices/shade-property-dialogs/GRKL300FinsCalibrationWizard>

dSS.devprops.CreatePanelForShadeProperties = function (record) {
    var matrix;
    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (!matrix.shadeprops) {
        Ext.Msg.alert(dSS._('Error'), dSS._('Shade properties can only be configured for grey klemme L devices'));
        return;
    }

    return dSS.util.isGrKl300TypeDevice(record)
        ? dSS.devprops.CreatePanelForGRKL300ShadeProperties(record, matrix)
        : dSS.devprops.CreatePanelForStandardShadeProperties(record, matrix);
}

dSS.devprops.CreatePanelForStandardShadeProperties = function (record, matrix) {
    var dsuid = record.get('dsuid');
    var shadeloaders = [
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_OPEN_CLOSE,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x32
            }
        }),
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_CLOSE_OPEN,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x30
            }
        }),
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_CALIBRATION_ALLOWED,
            url: '/json/device/getConfig',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3d
            }
        }),
        new dSS.devprops.Loader({             // advanced tab loader - TODO maybe move to common part (CreatePanelForShadeProperties)
            loaderId: dSS.devprops.Loaders.SHADE_MTIME_BLINK,
            url: '/json/device/getConfig',
            params:
            {
                dsuid: record.get('dsuid'),
                class: 3,
                index: 0x11
            }
        }),
        new dSS.devprops.Loader({             // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x38
            }
        }),
        new dSS.devprops.Loader({             // standard and advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3a
            }
        }),
        new dSS.devprops.Loader({         // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: record.get('dsuid'),
                class: 3,
                index: 0x36
            }
        }),
        new dSS.devprops.Loader({         // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_POS_TIME_MAX,
            url: '/json/device/getMaxMotionTime',
            params: {
                dsuid: dsuid
            }
        })
    ];

    if (matrix.motiontimefins) {
        shadeloaders.push(new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_MTIME_FINS,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3e
            }
        }));
    }

    var standardForms = [ {
        xtype: 'shadepropsform',
        dSUID: dsuid,
        matrix: matrix
    } ];

    var advancedForms = [ {
        xtype: 'shadepropsadvancedform',
        dSUID: dsuid,
        matrix: matrix
    } ];

    var forms = [
        {
            title: dSS._('Settings'),
            forms: standardForms
        },
        {
            title: dSS._('Advanced settings'),
            forms: advancedForms
        }
    ];

    return {
        panel: new dSS.devprops.DevicePropertiesMultitabPanel({
            DisplayID: record.get('DisplayID'),
            name: record.get('name'),
            showDefaults: true,
            tabForms: forms,
            loaders: shadeloaders
        }),
        extraTitle: dSS._('shade')
    };
};

dSS.devprops.CreatePanelForGRKL300ShadeProperties = function (record, matrix) {
    var dsuid = record.get('dsuid');
    var shadeloaders = [
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_OPEN_CLOSE,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x32
            }
        }),
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_CLOSE_OPEN,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x30
            }
        }),
        new dSS.devprops.Loader({             // standard and advanced tab loader
            loaderId: dSS.devprops.Loaders.ATTACHED_TERMINAL_DEVICE,
            url: '/api/v1/apartment/dsDevices/' + record.get('dsuid')
        }),
        new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.CALIBRATION_STATUS,             // standard and advanced tab loader
            url: '/json/device/getConfig',
            params: {
                dsuid: record.get('dsuid'),
                class: 1,
                index: 0x5c
            }
        }),
        new dSS.devprops.Loader({             // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x38
            }
        }),
        new dSS.devprops.Loader({             // standard and advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3a
            }
        }),
        new dSS.devprops.Loader({             // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_PTIME_FTUNE,
            url: '/json/device/getConfig',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3c
            }
        }),
        new dSS.devprops.Loader({         // standard and advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: record.get('dsuid'),
                class: 3,
                index: 0x36
            }
        }),
        new dSS.devprops.Loader({         // advanced tab loader
            loaderId: dSS.devprops.Loaders.SHADE_DEAD_TIME,
            url: '/json/device/getConfig',
            params: {
                dsuid: record.get('dsuid'),
                class: 3,
                index: 0x34
            }
        })
    ];

    if (matrix.motiontimefins) {        // advanced tab loader
        shadeloaders.push(new dSS.devprops.Loader({
            loaderId: dSS.devprops.Loaders.SHADE_MTIME_FINS,
            url: '/json/device/getConfigWord',
            params: {
                dsuid: dsuid,
                class: 3,
                index: 0x3e
            }
        }));
    }

    var standardForms = [ {
        xtype: 'grkl300shadepropsform',
        dSUID: dsuid,
        matrix: matrix
    } ];

    var advancedForms = [ {
        xtype: 'grkl300shadepropsadvancedform',
        dSUID: dsuid,
        matrix: matrix
    } ];

    var forms = [
        {
            title: dSS._('Settings'),
            forms: standardForms
        },
        {
            title: dSS._('Advanced settings'),
            forms: advancedForms
        }
    ];

    return {
        panel: new dSS.devprops.DevicePropertiesMultitabPanel({
            DisplayID: record.get('DisplayID'),
            name: record.get('name'),
            showDefaults: true,
            tabForms: forms,
            loaders: shadeloaders
        }),
        extraTitle: dSS._('shade')
    };
};
