/* globals dSS, Ext */

dSS.devprops.GRKL300ShadeCalibrationWizard = Ext.extend(Ext.Window, {
    constructor: function (config) {
        if (config.dSUID == null) {
            throw dSS._('Missing dSUID');
        }
        this.dSUID = config.dSUID;

        var devStore = dSS.data.Loader.getDeviceStore();
        var record = devStore.getById(this.dSUID);
        if (record == null) {
            throw dSS._('Device not found');
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var devClass = cls[0];

        if (devClass !== dSS.config.deviceClass.GR) {
            throw dSS._('Unsupported device group');
        }

        if (dSS.config.deviceTypes[type] !== 'KL') {
            throw dSS._('Unsupported device type');
        }

        this.attachedTerminalDevice = config.attachedTerminalDevice;
        var calibrationOptionLabels = dSS.config.AttachedShadeTerminalDeviceCalibrationOptionLabels[this.attachedTerminalDevice];
        var calibrationOptionImages = dSS.config.AttachedShadeTerminalDeviceImages[this.attachedTerminalDevice];

        var labelStyleToAlignWithInputs = { marginTop: 4 };

        this.cancelButton = new Ext.Button({
            cls: 'dsClsGRKL300ShadeCalibrationWizard_CancelButton',

            xtype: 'button',
            text: dSS._('Stop shade and cancel'),
            width: 200,
            handler: (function () {
                this.cancelCalibration();
                this.close();
            }).createDelegate(this)
        });

        this.closeButton = new Ext.Button({
            cls: 'dsClsGRKL300ShadeCalibrationWizard_CloseButton',

            text: dSS._('Close'),
            width: 80,
            handler: (function () {
                this.close();
            }).createDelegate(this),
            hidden: true
        });

        Ext.apply(this, {
            cls: 'dsClsGRKL300ShadeCalibrationWizard',

            layout: 'card',
            activeItem: 0,
            closable: false,
            padding: 10,
            width: 500,
            height: 300,
            resizable: false,
            modal: true,
            title: dSS._('Calibration - step 1'),
            items: [
                {
                    xtype: 'container',
                    ref: 'calibration',
                    layout: {
                        type: 'vbox',
                        defaultMargins: {
                            top: 0,
                            right: 5,
                            bottom: 0,
                            left: 0
                        },
                        padding: '5 0'
                    },
                    items: [
                        {
                            xtype: 'label',
                            ref: 'initialPosition',
                            text: dSS._(dSS.config.AttachedShadeTerminalDeviceCalibrationStep1Label[this.attachedTerminalDevice]) + ':',
                            style: labelStyleToAlignWithInputs,
                            height: 50,
                            hidden: false
                        },
                        {
                            cls: 'dsClsGRKL300ShadeCalibrationWizard_RadioGroup',

                            xtype: 'container',
                            ref: 'radioGroup',
                            height: 250,
                            items: [
                                {
                                    xtype: 'container',
                                    ref: 'radioGroupItemUp',
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'container',
                                            ref: 'wrapper',
                                            layout: 'hbox',
                                            flex: 1,
                                            items: [
                                                {
                                                    xtype: 'radio',
                                                    ref: 'radioButton',
                                                    name: 'position',
                                                    listeners: {
                                                        check: (function (self, checked) {
                                                            if (checked) {
                                                                this.handleUp();
                                                            }
                                                        }).createDelegate(this)
                                                    }
                                                },
                                                {
                                                    xtype: 'label',
                                                    text: dSS._(calibrationOptionLabels.up),
                                                    style: labelStyleToAlignWithInputs,
                                                    clickHandler: (function () {
                                                        this.calibration.radioGroup.radioGroupItemUp.wrapper.radioButton.setValue(true);
                                                    }).createDelegate(this),
                                                    listeners: {
                                                        afterrender: function (self) {
                                                            self.el.dom.addEventListener('click', this.clickHandler);
                                                        },
                                                        beforedestroy: function (self) {
                                                            self.el.dom.removeEventListener('click', this.clickHandler);
                                                        }
                                                    }
                                                }
                                            ]
                                        },
                                        {
                                            xtype: 'box',
                                            html: '<img src="/images/shades/' + calibrationOptionImages.up + '" alt="' + dSS._(calibrationOptionLabels.up) +
                                                '" width="60" />',
                                            width: 65,
                                            height: 80
                                        }
                                    ]
                                },
                                {
                                    xtype: 'container',
                                    ref: 'radioGroupItemDown',
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'container',
                                            ref: 'wrapper',
                                            layout: 'hbox',
                                            flex: 1,
                                            items: [
                                                {
                                                    xtype: 'radio',
                                                    ref: 'radioButton',
                                                    name: 'position',
                                                    listeners: {
                                                        check: (function (self, checked) {
                                                            if (checked) {
                                                                this.handleDown();
                                                            }
                                                        }).createDelegate(this)
                                                    }
                                                },
                                                {
                                                    xtype: 'label',
                                                    text: dSS._(calibrationOptionLabels.down),
                                                    style: labelStyleToAlignWithInputs,
                                                    clickHandler: (function () {
                                                        this.calibration.radioGroup.radioGroupItemDown.wrapper.radioButton.setValue(true);
                                                    }).createDelegate(this),
                                                    listeners: {
                                                        afterrender: function (self) {
                                                            self.el.dom.addEventListener('click', this.clickHandler);
                                                        },
                                                        beforedestroy: function (self) {
                                                            self.el.dom.removeEventListener('click', this.clickHandler);
                                                        }
                                                    }
                                                }
                                            ]
                                        },
                                        {
                                            xtype: 'box',
                                            html: '<img src="/images/shades/' + calibrationOptionImages.down + '" alt="' + dSS._(calibrationOptionLabels.down) +
                                                '" width="60" />',
                                            width: 65,
                                            height: 80
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    xtype: 'container',
                    ref: 'finalInfo',
                    items: [
                        {
                            xtype: 'label',
                            text: dSS._('Calibration process has started and will continue in background. You can close this window now.'),
                            style: {
                                padding: '5px 0'
                            }
                        }
                    ]
                }
            ],
            buttons: [
                this.cancelButton,
                this.closeButton
            ]
        });

        dSS.devprops.GRKL300ShadeCalibrationWizard.superclass.constructor.apply(this, arguments);
    },

    handleUp: function () {
        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 1,
                index: 0xfb,
                dsuid: this.dSUID,
                value: 0x11
            },
            failureMsg: dSS._('Could not calibrate'),
            maskEl: this.el,
            maskMsg: dSS._('Calibrating') + '...'
        },
        (function (success) {
            if (success) {
                this.layout.setActiveItem(1);
                this.setTitle(dSS._('Calibration - step 2'));
                this.closeButton.show();
                this.cancelButton.hide();
            } else {
                Ext.Msg.alert(dSS._('Could not calibrate'));
            }
        }).createDelegate(this));
    },

    handleDown: function () {
        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 1,
                index: 0xfb,
                dsuid: this.dSUID,
                value: 0xd0
            },
            failureMsg: dSS._('Could not calibrate'),
            maskEl: this.el,
            maskMsg: dSS._('Calibrating') + '...'
        },
        (function (success) {
            if (success) {
                this.layout.setActiveItem(1);
                this.setTitle(dSS._('Calibration - step 2'));
                this.closeButton.show();
                this.cancelButton.hide();
            } else {
                Ext.Msg.alert(dSS._('Could not calibrate'));
            }
        }).createDelegate(this));
    },

    cancelCalibration: function () {
        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 1,
                index: 0xfb,
                dsuid: this.dSUID,
                value: 0x57
            },
            failureMsg: dSS._('Could not stop calibration'),
            maskEl: this.el,
            maskMsg: dSS._('Stopping') + '...'
        },
        (function (success) {
            if (success) {
                this.close();
            } else {
                Ext.Msg.alert(dSS._('Could not stop calibration'));
            }
        }).createDelegate(this));
    }
});

Ext.reg('dssdevgrkl300shadecalibrationwizard', dSS.devprops.GRKL300ShadeCalibrationWizard);
