//= require <dSS/devices/shade-property-dialogs/ShadeCalibration>

/* globals dSS, Ext */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops', 'dSS.gthelper');
// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all data
// has been received
//
// parameters: dSUID, matrix

dSS.devprops.GRKL300ShadePropertiesForm = Ext.extend(dSS.devprops.BaseShadePropertiesForm, {
    constructor: function (config) {
        if (config.dSUID == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        if (config.matrix == null) {
            Ext.Msg.alert(dSS._('Error'), ('Missing matrix'));
            return;
        }

        this.dSUID = config.dSUID;
        this.matrix = config.matrix;

        this.statusPollingTimer = null;
        this.calibrationAllowed = null;
        this.calibrationStatusLoadedValue = null;
        this.deviceHasFins = null;
        this.reverseTimeOnDirectionChange = 0;
        this.motionTimeAndDirectionCalibrationRunning = false;

        this.buttonWidth = 80;
        this.columnWidths = [ 265, 170, 20 ];

        this.addEvents('formsubmitted', 'formloaded', 'forcereload', 'sharedevent', 'formretry', 'formchanged');

        this.calibrationNotAllowedReason = new Ext.form.Label({
            text: '',
            hidden: true,
            resizable: true,
            style: {
                display: 'block',
                paddingTop: '5px',
                paddingBottom: '5px',
                fontWeight: 'bold'
            }
        });

        var calibrateMotionTimeAndDirectionLabel = new Ext.form.Label({
            text: dSS._('Calibrate motion time and direction')
        });
        this.calibrateMotionTimeAndDirection = new Ext.Button({
            cls: 'dsClsGRKL300ShadePropertiesForm_StartMotionTImeAndDirectionCalibrationButton',

            text: dSS._('Start'),
            handler: this.startWizard.createDelegate(this),
            minWidth: this.buttonWidth,
            disabled: true
        });

        var calibrateMotionTimeAndDirectionStatusLabel = new Ext.form.Label({
            text: dSS._('Motion calibration status')
        });
        this.calibrateMotionTimeAndDirectionStatus = new Ext.form.Label({
            text: dSS._('loading') + '...'
        });

        this.calibrateTurnTimeLabel = new Ext.form.Label({
            text: dSS._('Calibrate turn time'),
            disabled: true
        });
        this.calibrateTurnTime = new Ext.Button({
            cls: 'dsClsGRKL300ShadePropertiesForm_StartTurnTimeCalibrationButton',

            text: dSS._('Start'),
            scope: this,
            handler: this.startFinsWizard.createDelegate(this),
            minWidth: this.buttonWidth,
            disabled: true
        });

        this.calibrateTurnTimeStatusLabel = new Ext.form.Label({
            text: dSS._('Turn time calibration status'),
            disabled: true
        });
        this.calibrateTurnTimeStatus = new Ext.form.Label({
            text: dSS._('loading') + '...',
            disabled: true
        });

        this.calibrationStop = new Ext.Button({
            cls: 'dsClsGRKL300ShadePropertiesForm_ResetButton',

            text: dSS._('Stop calibration'),
            disabled: true,
            handler: this.stopCalibration.createDelegate(this),
            minWidth: this.buttonWidth * 2 + 10
        });

        var items = [
            this.calibrationNotAllowedReason,
            this.makeRow([
                calibrateMotionTimeAndDirectionLabel,
                this.makeColumns([
                    this.calibrateMotionTimeAndDirection
                ])
            ]),
            this.makeRow([
                calibrateMotionTimeAndDirectionStatusLabel,
                this.calibrateMotionTimeAndDirectionStatus
            ], '5 0 25'),
            this.makeRow([
                this.calibrateTurnTimeLabel,
                this.makeColumns([
                    this.calibrateTurnTime
                ])
            ]),
            this.makeRow([
                this.calibrateTurnTimeStatusLabel,
                this.calibrateTurnTimeStatus
            ], '5 0 25'),
            this.makeRow([
                null,
                this.calibrationStop
            ])
        ];

        Ext.apply(this, {
            cls: 'dsClsGRKL300ShadePropertiesForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'forcereload', 'formretry', 'formchanged' ],
            padding: 5,
            scope: this,
            trackResetOnLoad: true,
            items: items
        });

        this.resetLoaded();

        dSS.devprops.GRKL300ShadePropertiesForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function (data) {
            // TODO add error handling for timeout - not sure if this method's gonna get called
            if (this.closed) {
                return;
            }

            var val;
            var ok = Boolean(data && data.json && data.json.ok);
            if (ok) {
                val = data.json.result && data.json.result.value;
            }

            switch (data.loader) {
                case dSS.devprops.Loaders.ATTACHED_TERMINAL_DEVICE:
                    this.loaded.attachedTerminalDevice = Boolean(data && data.json && data.json.data);
                    if (this.loaded.attachedTerminalDevice) {
                        this.attachedTerminalDevice = data.json.data.attributes.shadeAttributes.shadeType;
                        this.deviceHasFins = [
                            dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS,
                            dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS
                        ].indexOf(this.attachedTerminalDevice) !== -1;
                    } else {
                        this.attachedTerminalDevice = null;
                    }
                    if (this.deviceHasFins) {
                        this.calibrateTurnTimeLabel.enable();
                        this.calibrateTurnTimeStatusLabel.enable();
                        this.calibrateTurnTimeStatus.enable();
                    }
                    if (this.loaded.calibrateStatus != null) {
                        this.setCalibrationStatus(this.calibrationStatusLoadedValue);
                    }
                    break;

                case dSS.devprops.Loaders.CALIBRATION_STATUS:
                    this.loaded.calibrateStatus = ok;
                    this.calibrationStatusLoadedValue = val;
                    if (this.loaded.shadeFtEndThreshold != null) { // not needed, but added for readibility
                        if (ok) {
                            this.setCalibrationStatus(val);
                        } else {
                            this.setCalibrationStatusToUnknown();
                        }
                    }
                    break;

                case dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE:
                    this.loaded.numStepsMTimeFinetune = ok;
                    if (ok) {
                        this.numSteps = this.lowerByte(val);
                        this.mTimeAdjust = this.higherByte(val);
                    } else {
                        this.numSteps = null;
                        this.mTimeAdjust = null;
                    }
                    break;

                case dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW:
                    this.loaded.shadeFtEndThreshold = ok;
                    if (ok) {
                        var threshold = this.higherByte(val);
                        this.calibrationAllowed = threshold !== 0;
                        if (this.calibrationAllowed) {
                            this.calibrationNotAllowedReason.hide();
                            this.calibrationNotAllowedReason.setText('');
                        } else {
                            this.calibrationNotAllowedReason.setText(dSS._('Automatic calibration impossible due to threshold of end switch recognition being 0. Calibrate the device manually by configuring values in Advanced tab.'));
                            this.calibrationNotAllowedReason.show();
                        }
                    } else {
                        this.calibrationAllowed = false;
                        this.calibrationNotAllowedReason.setText(dSS._('Calibration impossible due to threshold of end switch recognition value failed to load. Try reopening this dialog.'));
                        this.calibrationNotAllowedReason.show();
                    }

                    // a hack to resize window after showing/hiding calibrationNotAllowedReason label
                    var panelBody = document.querySelector('.dsClsDevicePropertiesMultitabPanel .x-panel-body');
                    panelBody.style.height = '';
                    Ext.getCmp('dsClsDevicePropertiesWindow').fireEvent('updateshadow');

                    if (this.loaded.calibrateStatus != null) {
                        this.setCalibrationStatus(this.calibrationStatusLoadedValue);
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                    this.loaded.reverseTimeOnDirectionChange = ok;
                    if (ok) {
                        var rt = this.higherByte(val);
                        this.reverseTimeOnDirectionChange = rt * 10;
                    }
                    break;

                default:
                    return;
            }

            if (this.isAllLoaded()) {   // custom allLoaded in this component
                this.fireEvent('formloaded', true);
                if (Object.values(this.loaded).filter(function (ok) {
                    return !ok;
                }).length) {
                    Ext.Msg.alert(dSS._('Warning'), dSS._('Some data did not load correctly and the dialog values may not reflect device configuration. Please try again after a while.'));
                    if (!this.loaded.calibrateStatus) {
                        this.setCalibrationStatusToUnknown();
                    }
                }
                this.resetLoaded();
                if (this.motionTimeAndDirectionCalibrationRunning) {
                    if (this.motionCalibrationDone()) {
                        this.motionTimeAndDirectionCalibrationRunning = false;
                        if (this.deviceHasFins && !this.turnTimeCalibrationDone()) {
                            this.promptForTurnTimeCalibration();
                        }
                    }
                }
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
        }, this);

        this.on('submit', function () {
            if (!this.getForm().isValid()) {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Please correct values in the red marked fields and try again')
                );
                this.fireEvent('formretry');
                return;
            }

            this.saveOK = true;
            var call = this.saveData.createDelegate(this, [ [], true ]);
            call();
        }, this);

        this.on('sharedevent', function (name, value) {
            if (name === 'advancedvalueschanged') {
                this.handleChangeInAdvancedTab(value);
            }
        }, this);
    },

    isAllLoaded () {
        var that = this;
        return Object.keys(this.loaded)
            .reduce(function (allLoaded, item) {
                return allLoaded && that.loaded[item] != null;
            }, true);
    },

    higherByte: function (value) {
        return (value >> 8) & 0x000000ff;
    },
    lowerByte: function (value) {
        return value & 0x000000ff;
    },

    motionCalibrationDone () {
        var motionCalibrationStatus = dSS.devprops.ShadeCalibration.motionAndDirectionStatus;
        return this.motionTimeAndDirectionCalibrationStatus === motionCalibrationStatus.AUTOMATICALLY_CALIBRATED.value ||
            this.motionTimeAndDirectionCalibrationStatus === motionCalibrationStatus.MANUALLY_CALIBRATED.value;
    },
    turnTimeCalibrationDone () {
        var turnTimeCalibrationStatus = dSS.devprops.ShadeCalibration.turnTimeCalibrationStatus;
        return this.motionTimeAndDirectionCalibrationStatus === turnTimeCalibrationStatus.MANUALLY_CALIBRATED.value;
    },
    enableCalibration () {
        if (this.calibrationAllowed) {
            this.calibrateMotionTimeAndDirection.enable();
            if (this.deviceHasFins && this.motionCalibrationDone()) {
                this.calibrateTurnTime.enable();
            }
        }
    },
    disableCalibration () {
        this.calibrateMotionTimeAndDirection.disable();
        this.calibrateTurnTime.disable();
    },
    setCalibrationStatus (parameterValue) {
        var helpers = dSS.devprops.ShadeCalibration;
        var previousMotionTimeAndDirectionCalibrationStatus = this.motionTimeAndDirectionCalibrationStatus;
        this.motionTimeAndDirectionCalibrationStatus = helpers.getMotionAndDirectionCalibrationStatus(parameterValue);  // lower nibble
        this.turnTimeCalibrationStatus = helpers.getTurnTimeCalibrationStatus(parameterValue);  // higher nibble

        this.calibrateMotionTimeAndDirectionStatus.setText(
            helpers.getStatusLabel('motionAndDirectionStatus', this.motionTimeAndDirectionCalibrationStatus, this.attachedTerminalDevice) || '?'
        );
        if (this.motionTimeAndDirectionCalibrationStatus === dSS.devprops.ShadeCalibration.motionAndDirectionStatus.NEVER_CALIBRATED.value) {
            this.calibrateMotionTimeAndDirectionStatus.addClass('invalid-msg');
        } else {
            this.calibrateMotionTimeAndDirectionStatus.removeClass('invalid-msg');
        }

        this.calibrateTurnTimeStatus.setText(
            helpers.getStatusLabel('turnTimeCalibrationStatus', this.turnTimeCalibrationStatus, this.attachedTerminalDevice) || '?'
        );
        if (this.turnTimeCalibrationStatus === dSS.devprops.ShadeCalibration.turnTimeCalibrationStatus.NEVER_CALIBRATED.value) {
            this.calibrateTurnTimeStatus.addClass('invalid-msg');
        } else {
            this.calibrateTurnTimeStatus.removeClass('invalid-msg');
        }

        var me = this;
        var isRunning = helpers.running.find(function (running) {
            return running.value === me.motionTimeAndDirectionCalibrationStatus;
        });
        if (isRunning) {
            this.disableCalibration();
            this.calibrationStop.enable();
            this.statusPollingTimer = setTimeout(this.pollStatus.createDelegate(this), 2000);
            this.disableAdvancedTab();
        } else {
            this.enableCalibration();
            this.calibrationStop.disable();
            this.stopPollingStatus();
            this.enableAdvancedTab();
            var wasRunning = helpers.running.find(function (running) {
                return running.value === previousMotionTimeAndDirectionCalibrationStatus;
            });
            if (wasRunning) { // for when calibration was completed and Advanced tab values need to be reloaded
                this.reloadData();
            }
        }
    },
    setCalibrationStatusToUnknown () {
        this.calibrateMotionTimeAndDirectionStatus.setText('?');
        this.calibrateMotionTimeAndDirectionStatus.removeClass('invalid-msg');
        this.calibrateTurnTimeStatus.setText('?');
        this.calibrateTurnTimeStatus.removeClass('invalid-msg');
        this.enableCalibration();
        this.calibrationStop.disable();
        this.statusPollingTimer = setTimeout(this.pollStatus.createDelegate(this), 2000);
        this.enableAdvancedTab();
    },
    setCalibrationStatusToLoading () {
        this.calibrateMotionTimeAndDirectionStatus.setText(dSS._('loading') + '...');
        this.calibrateMotionTimeAndDirectionStatus.removeClass('invalid-msg');
        this.calibrateTurnTimeStatus.setText(dSS._('loading') + '...');
        this.calibrateTurnTimeStatus.removeClass('invalid-msg');
        this.disableCalibration();
        this.calibrationStop.disable();
        this.disableAdvancedTab();
    },
    pollStatus () {
        this.stopPollingStatus();
        var scope = this;
        dSS.util.genericRequest({
            url: '/json/device/getConfig',
            scope: this,
            params: {
                class: 1,
                index: 0x5c,
                dsuid: this.dSUID
            },
            failureMsg: dSS._('Could not poll status')
        },
        function (success, response) {
            if (this.closed) {
                return;
            }
            if (success && response && response.ok) {
                scope.setCalibrationStatus(response.result.value);
            } else {
                scope.setCalibrationStatusToUnknown();
            }
        });
    },
    stopPollingStatus () {
        if (this.statusPollingTimer) {
            clearTimeout(this.statusPollingTimer);
        }
        this.statusPollingTimer = null;
    },
    disableAdvancedTab () {
        this.ownerCt.ownerCt.items.items[1].disable();
    },
    enableAdvancedTab () {
        this.ownerCt.ownerCt.items.items[1].enable();
    },
    promptForTurnTimeCalibration () {
        var startFinsWizard = this.startFinsWizard.createDelegate(this);

        var callback = function (button) {
            if (button === 'yes') {
                startFinsWizard();
            }
        };

        Ext.Msg.show({
            title: dSS._('Continue calibration?'),
            msg: dSS._('It is recommended to complete calibration of the device by running turn time calibration wizard. Do you want to start it now?'),
            buttons: {
                yes: dSS._('Yes'),
                no: dSS._('No')
            },
            fn: callback
        });
    },

    resetLoaded () {
        this.loaded = {
            attachedTerminalDevice: null,   // null - not loaded, true - loaded ok, false - load failure
            calibrateStatus: null,
            numStepsMTimeFinetune: null,
            shadeFtEndThreshold: null,
            reverseTimeOnDirectionChange: null
        };
        this.calibrationStatusLoadedValue = null;
    },

    handleChangeInAdvancedTab: function (dirty) {
        if (dirty) {
            this.disableCalibration();
            this.calibrationTips = [
                new Ext.ToolTip({
                    target: this.calibrateMotionTimeAndDirection.getEl(),
                    html: dSS._('Please save changes in Advanced tab before starting calibration.')
                }),
                new Ext.ToolTip({
                    target: this.calibrateTurnTime.getEl(),
                    html: dSS._('Please save changes in Advanced tab before starting calibration.')
                })
            ];
        } else {
            this.enableCalibration();
            this.calibrationTips.forEach(function (tip) {
                tip.destroy();
            });
        }
    },

    saveData: function () { // records, ok
        this.fireEvent('formsubmitted');
    },

    startWizard: function () {
        var dSUID = this.dSUID;

        var reload = this.reloadData.createDelegate(this);
        var scope = this;

        this.disableCalibration();
        var initialCalibrationStatus = this.calibrateMotionTimeAndDirectionStatus.text;

        var callback = function () {
            scope.on('dataloaded', function () {
                // reenable save button that is reset by the
                // reload action, #13291
                scope.fireEvent('formchanged');
            }, scope, { scope: scope, single: true });
            reload();
        };

        this.motionTimeAndDirectionCalibrationRunning = true;

        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);     // send STOP scene

        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 1,
                index: 0xfb,
                dsuid: this.dSUID,
                value: 0x5e         // write SETUP
            },
            failureMsg: dSS._('Could not start calibration'),
            maskEl: this.ownerCt.el,
            maskMsg: dSS._('Loading') + '...'
        },
        function (success) {
            if (success) {
                try {
                    var wizard = new dSS.devprops.GRKL300ShadeCalibrationWizard({
                        dSUID: dSUID,
                        attachedTerminalDevice: scope.attachedTerminalDevice
                    });

                    wizard.on('close', callback);
                    wizard.show();
                } catch (error) {
                    Ext.Msg.alert(dSS._('Error'), error);
                    this.motionTimeAndDirectionCalibrationRunning = false;
                    scope.calibrateMotionTimeAndDirection.enable();
                    if (scope.motionCalibrationDone()) {
                        scope.calibrateTurnTime.enable();
                    }
                    scope.calibrateMotionTimeAndDirectionStatus.setText(initialCalibrationStatus);
                    scope.calibrationStop.disable();
                }
            } else {
                this.motionTimeAndDirectionCalibrationRunning = false;
                scope.calibrateMotionTimeAndDirection.enable();
                if (scope.motionCalibrationDone()) {
                    scope.calibrateTurnTime.enable();
                }
                scope.calibrateMotionTimeAndDirectionStatus.setText(initialCalibrationStatus);
                scope.calibrationStop.disable();
            }
        });
    },

    startFinsWizard: function () {
        var dSUID = this.dSUID;

        var reload = this.reloadData.createDelegate(this);
        var scope = this;

        this.disableCalibration();
        var initialCalibrationStatus = this.calibrateTurnTimeStatus.text;
        this.calibrateTurnTimeStatus.setText(dSS._('Calibration running'));
        this.calibrationStop.enable();

        var callback = function () {
            scope.on('dataloaded', function () {
                // reenable save button that is reset by the
                // reload action, #13291
                scope.fireEvent('formchanged');
            }, scope, { scope: scope, single: true });
            reload();
        };

        try {
            var wizard = new dSS.devprops.GRKL300FinsCalibrationWizard({
                dSUID: dSUID,
                mtimeadjust: this.mTimeAdjust,
                motionTimeAndDirectionCalibrationStatus: this.motionTimeAndDirectionCalibrationStatus,
                reverseTimeOnDirectionChange: this.reverseTimeOnDirectionChange
            });
            wizard.on('close', callback);
            wizard.show();
        } catch (error) {
            Ext.Msg.alert(dSS._('Error'), error);
            this.calibrateTurnTimeStatus.setText(initialCalibrationStatus);
            this.enableCalibration();
            this.calibrationStop.disable();
        }
    },

    reloadData: function () {
        this.resetLoaded();
        this.setCalibrationStatusToLoading();
        this.disableAdvancedTab();
        this.fireEvent('forcereload');
    },

    stopCalibration: function () {
        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 1,
                index: 0xfb,
                dsuid: this.dSUID,
                value: 0x57
            },
            failureMsg: dSS._('Could not stop calibration'),
            maskEl: this.ownerCt.el,
            maskMsg: dSS._('Stopping') + '...'
        },
        (function (success) {
            if (success) {
                this.loaded.calibrateStatus = null;
                this.motionTimeAndDirectionCalibrationRunning = false;
                this.setCalibrationStatusToLoading();
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CALIBRATION_STATUS,
                    url: '/json/device/getConfig',
                    params: {
                        dsuid: this.dSUID,
                        class: 1,
                        index: 0x5c
                    },
                    listeners: {
                        loadcomplete: (function (data) {
                            this.loaded.calibrateStatus = Boolean(data && data.json && data.json.ok);
                            if (data && data.json && data.json.ok && data.json.result) {
                                this.setCalibrationStatus(data.json.result.value);
                            } else {
                                this.setCalibrationStatusToUnknown();
                            }
                        }).createDelegate(this)
                    }
                }).load();
            } else {
                Ext.Msg.alert(dSS._('Could not stop calibration'));
            }
        }).createDelegate(this));
    }
});

Ext.reg('grkl300shadepropsform', dSS.devprops.GRKL300ShadePropertiesForm);
