Ext.namespace('dSS', 'dSS.grid', "dSS.gthelper");

dSS.grid.BinaryInputsPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {

        var BinaryInputTypeRenderer = function(value, metaData, record,
                                               rowIndex, colIndex, store) {
            var input = parseInt(record.get('inputType'));
            if (input === NaN) {
                return record.get('inputType');
            }

            var type = dSS.config.AKMInputTypeStrings[input];
            if (type === undefined) {
                type = record.get('inputType');
            } else {
                type = dSS._(type);
            }

            return type;
        };

        var StateValueRenderer = function(value, metaData, record, rowIndex,
                                          colIndex, store) {
            var state = parseInt(record.get('stateValue'));
            if (state === NaN) {
                state = 0;
            }

            var stateString = dSS.config.StateValueStrings[state];
            if (stateString === undefined) {
                stateString = record.get('stateValue');
            } else {
                stateString = dSS._(stateString);
            }

            return stateString;
        };

        var columns =
        [
            {
                header: dSS._("Index"),
                dataIndex: 'inputIndex'
            },
            {
                header: dSS._("Input type"),
                dataIndex: 'inputType',
                renderer: BinaryInputTypeRenderer
            },
            {
                header: dSS._("Input id"),
                dataIndex: 'inputId'
            },
            {
                header: dSS._("Input state"),
                dataIndex: 'stateValue',
                renderer: StateValueRenderer
            }
        ];

        var BinaryInputRecord = Ext.data.Record.create(
        [
            { name: 'inputIndex' },
            { name: 'inputType' },
            { name: 'inputId' },
            { name: 'stateValue' }
        ]);

        var BinaryInputReader = new Ext.data.JsonReader(
            {
                root: 'binaryInputs',
                idProperty: 'inputIndex'
            },
            BinaryInputRecord
        );

        Ext.apply(this, {
            cls: 'dsClsBinaryInputsPanel',

            title: dSS._("Binary inputs"),
            store: new Ext.data.Store({
                reader: BinaryInputReader,
                sortInfo: {
                    field: 'inputIndex',
                    direction: 'ASC'
                }
            }),
            columns: columns,
            stripeRows: true,
            selModel: new Ext.grid.RowSelectionModel({ singleSelect: true }),
            viewConfig: {
                forceFit: true
            }
        });
        dSS.grid.BinaryInputsPanel.superclass.initComponent.apply(this, arguments);

        dSS.MessageBus.on('loadbinaryinputdata',
                this.loadBinaryInputData.createDelegate(this), this);
        dSS.MessageBus.on('clearbinaryinputdata',
                this.clearBinaryInputData.createDelegate(this), this);
    },
    loadBinaryInputData: function(dsuid) {
        Ext.Ajax.request({
            url: '/json/property/query',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                query: '/apartment/zones/zone0/devices/' + dsuid +
                       '/binaryInputs/*(inputType,inputId,inputIndex,stateValue)',
            },
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (json.ok) {
                        this.getStore().loadData(json.result);
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not load binary input " +
                                        "information from dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                          dSS._("Failure when requesting binary input " +
                                "information from dSS"));
            }
        });
    },
    clearBinaryInputData: function() {
        this.getStore().removeAll();
    }
});

Ext.reg('dssbinaryinputspanel', dSS.grid.BinaryInputsPanel);
