Ext.namespace('dSS', 'dSS.system');

dSS.system.jsLogFilesPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        this.logFileRecord = Ext.data.Record.create([
        {
            name: "logfile",
            mapping: "name"
        },
        {
            name: "download",
            mapping: "download"
        }]);

        this.logFileStore = new Ext.data.ArrayStore({
            storeId: "logFileStore",
            fields: this.logFileRecord
        });

        Ext.apply(this, {
            cls: 'dsClsJsLogFilesPanel',

            columns: [
            {
                header: dSS._("Log file"),
                sortable: true,
                dataIndex: "name",
                id: "logfile"
            },
            {
                header: dSS._("Download"),
                sortable: true,
                dataIndex: "download",
                width: 200
            }],
            store: this.logFileStore,
            stripeRows: true,
            autoScroll: true,
            autoExpandColumn: "logfile",
            layout: "fit",
            selModel: new Ext.grid.RowSelectionModel({
                singleSelect: true
            }),
            viewConfig: {
                autoFill: true
            }
        });

        dSS.system.jsLogFilesPanel.superclass.initComponent.apply(this, arguments);

        this.on("afterrender", function(component) {
            this.fillStore();
        },
        this);

        this.on("activate", function(component) {
            this.fillStore();
            var sel = this.getSelectionModel();
            if (sel.hasSelection()) {
                var record = sel.getSelected();
                this.findParentByType("dsssystemjslogfiles").loadLog(record.get("name"));
            }
        },
        this);

        this.on("cellclick", function(grid, rowIndex, columnIndex, e) {
            var record = grid.getStore().getAt(rowIndex);
            this.findParentByType("dsssystemjslogfiles").loadLog(record.get("name"));
        },
        this);
    },

    fillStore: function() {
        Ext.Ajax.request({
            url: "/json/property/getChildren",
            method: "GET",
            scope: this,
            params: {
                path: "/system/js/logfiles"
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        for (i = 0; i < jsonData.result.length; i++) {
                            var link = "<span class=\"x-green-button-link\">" + "<a target=\"_blank\" title=\"" + dSS._("Right click and select Save Link As to " + "download the log file") + "\" href=\"/download/" + jsonData.result[i].name + "\">" + jsonData.result[i].name + "</a></span>";
                            jsonData.result[i].download = link;
                            var r = new this.logFileRecord(
                            jsonData.result[i]);

                            if (this.logFileStore.findExact("name", r.get("name")) === - 1) {
                                this.logFileStore.add([r]);
                            }
                        }
                    } else {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not get properties from dSS"));
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send getChildren command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send getChildren command to dSS"));
            }
        });
    }
});

Ext.reg('dssjslogfilespanel', dSS.system.jsLogFilesPanel);

