//= require <dSS/tree/PropertyTreeLoader>
Ext.namespace('dSS', 'dSS.system');

dSS.system.PropertyTree = Ext.extend(Ext.Panel, {
    initComponent: function() {
        this.contextMenu = null;

        Ext.apply(this, {
            cls: 'dsClsPropertyTree',

            layout: 'border',
            items: [
            {
                xtype: 'treepanel',
                ref: 'propertytree',
                region: 'center',
                width: 225,
                // give east and west regions a width
                minSize: 175,
                maxSize: 400,
                autoScroll: true,
                animate: true,
                enableDD: false,
                containerScroll: true,
                border: false,
                loader: new dSS.tree.PropertyTreeLoader(),
                root: new Ext.tree.AsyncTreeNode({
                    cls: 'dsClsPropertyTree_Root',

                    expanded: false,
                    path: '/',
                    type: 'none',
                    text: 'dSS',
                    leaf: false
                }),
                rootVisible: true,
                listeners: {
                    append: this.handleAppend,
                    contextmenu: this.handleContextMenu.createDelegate(this)
                }
            }]
        });

        dSS.system.PropertyTree.superclass.initComponent.apply(this, arguments);
    },
    handleAppend: function(tree, parent, node, index) {
        if (node.attributes.type === 'none') {
            return;
        }
        var url = "/json/property/";
        switch (node.attributes.type) {
        case 'string':
            url += 'getString';
            break;
        case 'integer':
            url += 'getInteger';
            break;
        case 'boolean':
            url += 'getBoolean';
            break;
        case 'floating':
            url += 'getFloating';
            break;
        default:
            return;
        }
        Ext.Ajax.request({
            url:
            url,
            params: {
                path: node.attributes.path
            },
            method: 'GET',
            success: function(response, opts) {
                var obj = Ext.decode(response.responseText);
                if (obj.ok === true) {
                    node.setText(node.text + " : " + obj.result.value);
                }
            },
            failure: function(response, opts) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Server side failure with status code") + ": " + response.status);
            }
        });
    },
    handleContextMenu: function (node, event) {
        event.preventDefault();
        event.stopEvent();

        if (!this.contextMenu) {
            this.contextMenu = new Ext.menu.Menu({
                cls: 'dsClsPropertyTree_ContextMenu'
            });
        } else {
            this.contextMenu.removeAll();
        }

        var menuItem = new Ext.menu.Item({
            cls: 'dsClsPropertyTree_ContextMenu_CopyPath',

            text: dSS._('Copy node path'),
            iconCls: 'copyToClipboardIcon',
            handler: this.copyPath.createDelegate(this, [ node, false ])
        });
        this.contextMenu.add(menuItem);

        menuItem = new Ext.menu.Item({
            cls: 'dsClsPropertyTree_ContextMenu_CopyNode',

            text: dSS._('Copy node value'),
            iconCls: 'copyToClipboardIcon',
            handler: this.copyNodeValue.createDelegate(this, [ node, false ])
        });
        this.contextMenu.add(menuItem);

        menuItem = new Ext.menu.Item({
            cls: 'dsClsPropertyTree_ContextMenu_CopyTree',

            text: dSS._('Copy tree'),
            iconCls: 'copyToClipboardIcon',
            handler: this.copyTree.createDelegate(this, [ node, false ])
        });
        this.contextMenu.add(menuItem);

        var path = node.getPath().split('/').slice(1);

        var xy = event.getXY();
        this.contextMenu.showAt(xy);
    },

    copyPath: function (node) {
        this.copyToClipboard(node.attributes.path);
    },
    copyNodeValue: function (node) {
        var text = node.text.split(' : ');
        this.copyToClipboard(text[1] != null ? text[1] : text[0]);
    },
    copyTree: function (node) {
        var path = node.attributes.path.split('/').slice(1, -1);
        var text = node.text.split(' : ');
        var tree = {};
        var currentNode = tree;
        path.forEach(function (part) {
            currentNode[part] = {};
            currentNode = currentNode[part];
        });
        if (node.isLeaf()) {
            currentNode[text[0]] = text[1];
        } else {
            function parseForest (forest) {
                return forest.reduce(function (json, tree) {
                    var childText = tree.text.split(' : ');
                    if (tree.isLeaf()) {
                        json[childText[0]] = childText[1];
                    } else {
                        json[childText[0]] = parseForest(tree.childNodes);
                    }
                    return json;
                }, {});
            }
            currentNode[text[0]] = parseForest(node.childNodes);
        }
        this.copyToClipboard(JSON.stringify(tree, null, 4));
    },

    copyToClipboard: function (content) {
        const clipboardEl = document.createElement('textarea');
        clipboardEl.innerHTML = content;
        document.body.append(clipboardEl);
        clipboardEl.select();
        try {
            document.execCommand('copy');
        } catch (err) {
            console.log('Could not copy content to clipboard.');
        }
        clipboardEl.remove();
    }
});

Ext.reg('dsssystempropertytree', dSS.system.PropertyTree);
